@@ -9,17 +9,19 @@
  *                                                                         *
  * Functions    : MdcLoadLUT()       - Load LUT file into RGB array        *
  *                MdcGrayScale()     - Make gray palette                   *
+ *                MdcInvertedScale() - Make inverted gray palette          *
  *                MdcRainbowScale()  - Make rainbow palette                *
  *                MdcCombinedScale() - Make combined palette               *
  *                MdcHotmetalScale() - Make hotmetal palette               *
  *                MdcGetColorMap()   - Get the specified palette           *
+ *                MdcSetPresentMap() - Preserve colormap in colored file   *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-color.c,v 1.1.1.1 2000/10/28 16:50:57 enlf Exp $
+/* $Id: m-color.c,v 1.12 2004/01/04 22:36:16 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -39,15 +41,27 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
 #include <stdio.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
 #include "m-defs.h"
 #include "m-global.h"
+#include "m-color.h"
 
 /****************************************************************************
                               D E F I N E S 
 ****************************************************************************/
 
 static Uint8 loaded_map[768], LOADED = MDC_NO;
+static Uint8 present_map[768];            /* map from file */
 
 /* cti source */
 struct {int n,r,g,b,dr,dg,db; }
@@ -90,19 +104,26 @@ void MdcGrayScale(Uint8 *palette)
   int i;
   Uint8 gray;
    
-
   for (i=0; i<256; i++) {
-     if (MDC_CMAP_INVERS == MDC_YES) {
-       gray = 255 - (Uint8)i;
-     }else{
-       gray = (Uint8)i;
-     }
-  
+     gray = (Uint8)i;
      palette[i*3]=palette[i*3+1]=palette[i*3+2]=gray;
   }
 
 }
  
+void MdcInvertedScale(Uint8 *palette)
+{
+  int i;
+  Uint8 gray;
+   
+
+  for (i=0; i<256; i++) {
+     gray = 255 - (Uint8)i;
+     palette[i*3]=palette[i*3+1]=palette[i*3+2]=gray;
+  }
+
+}
+
 void MdcRainbowScale(Uint8 *palette)
 {   
   int p=0,i,j,r,g,b;
@@ -186,10 +207,12 @@ void MdcGetColorMap(int map, Uint8 palette[])
 {
 
   switch (map) {
-    case MDC_MAP_PRESENT : /* do nothing */
+    case MDC_MAP_PRESENT : memcpy(palette,present_map,768);
                       break;
     case MDC_MAP_GRAY    : MdcGrayScale(palette); 
                       break;
+    case MDC_MAP_INVERTED: MdcInvertedScale(palette);
+                      break;
     case MDC_MAP_RAINBOW : MdcRainbowScale(palette);
                       break;
     case MDC_MAP_COMBINED: MdcCombinedScale(palette);
@@ -199,7 +222,14 @@ void MdcGetColorMap(int map, Uint8 palette[])
     case MDC_MAP_LOADED  : 
                       if (LOADED == MDC_YES) memcpy(palette,loaded_map,768);
                       break;
-    default: MdcGrayScale(palette); 
+    default: MdcGrayScale(palette);
   }
 
 }
+
+int MdcSetPresentMap(Uint8 palette[])
+{
+  memcpy(present_map,palette,768);
+
+  return(MDC_MAP_PRESENT);
+}
