@@ -7,16 +7,18 @@
  *                                                                         *
  * project      : (X)MedCon by Erik Nolf                                   *
  *                                                                         *
- * Functions    : MdcPrntWarn()         - Print a warning                  *
+ * Functions    : MdcPrntStream()       - Gives proper output stream       *
+ *                MdcPrntScrn()         - Print to screen                  *
  *                MdcPrntMesg()         - Print a message                  *
+ *                MdcPrntWarn()         - Print a warning                  *
  *                MdcPrntErr()          - Print error and leave            *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-error.c,v 1.1.1.1 2000/10/28 16:50:58 enlf Exp $
+/* $Id: m-error.c,v 1.18 2004/01/04 22:36:16 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -36,64 +38,102 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <stdarg.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 
 #include "m-defs.h"
 #include "m-global.h"
+#include "m-error.h"
 
-/****************************************************************************
-                              D E F I N E S 
-****************************************************************************/
-
-void MdcPrntWarn(char *fmt, ...);
-void MdcPrntMesg(char *fmt, ...);
-void MdcPrntErr(int code, char *fmt, ...);
+#if GLIBSUPPORTED
+#include <glib.h>
+#endif
 
 /****************************************************************************
                             F U N C T I O N S
 ****************************************************************************/
 
-void MdcPrntWarn(char *fmt, ...)
-{
-  va_list args; 
+static FILE *MdcPrntStream(void)
+{ 
+  FILE *stream;
 
-  fprintf(stdout,"\n%s: Warning: ",MDC_PRGR);
+  if (MDC_FILE_STDOUT == MDC_YES) {
+    stream = stderr;
+  }else{
+    stream = stdout;
+  }
+  return(stream);
+}
+
+
+void MdcPrntScrn(char *fmt, ...)
+{
+  va_list args;
 
   va_start(args, fmt);
-  vsprintf(errmsg, fmt, args);
-  fprintf(stdout,"%s\n\n",errmsg);
+  vfprintf(MdcPrntStream(), fmt, args);
   va_end(args);
-
 }
 
 void MdcPrntMesg(char *fmt, ...)
 {
   va_list args;
 
-  fprintf(stdout,"\n%s: Message: ",MDC_PRGR);
+  if (MDC_BLOCK_MESSAGES < MDC_LEVEL_MESG) {
+    va_start(args,fmt);
 
-  va_start(args,fmt);
-  vsprintf(errmsg, fmt, args);
-  fprintf(stdout,"%s\n\n",errmsg);
-  va_end(args);
+#if GLIBSUPPORTED
+    g_logv(MDC_PRGR,G_LOG_LEVEL_MESSAGE, fmt, args);
+#else
+    MdcPrntScrn("\n%s: Message: ",MDC_PRGR);
+    vfprintf(MdcPrntStream(), fmt, args); fprintf(MdcPrntStream(),"\n\n");
+#endif
 
+    va_end(args);
+  }
 }
 
+void MdcPrntWarn(char *fmt, ...)
+{
+  va_list args; 
+
+  if (MDC_BLOCK_MESSAGES < MDC_LEVEL_WARN) {
+    va_start(args, fmt);
+
+#if GLIBSUPPORTED
+    g_logv(MDC_PRGR,G_LOG_LEVEL_WARNING, fmt, args);
+#else
+    MdcPrntScrn("\n%s: Warning: ",MDC_PRGR);
+    vfprintf(MdcPrntStream(), fmt, args); fprintf(MdcPrntStream(),"\n\n");
+#endif
+
+    va_end(args);
+  }
+}
 
 void MdcPrntErr(int code, char *fmt, ...)
 {
   va_list args;
 
-  fprintf(stderr,"\n%s: Error  : ",MDC_PRGR);
+  if (MDC_BLOCK_MESSAGES < MDC_LEVEL_ERR) {
+    va_start(args, fmt);
 
-  va_start(args, fmt);
-  vsprintf(errmsg, fmt, args);
-  fprintf(stderr,"%s\n\n",errmsg);
-  va_end(args);
+#if GLIBSUPPORTED
+    g_logv(MDC_PRGR,G_LOG_LEVEL_ERROR, fmt, args);
+#else
+    MdcPrntScrn("\n%s: Error  : ",MDC_PRGR);
+    vfprintf(MdcPrntStream(), fmt, args); fprintf(MdcPrntStream(),"\n\n");
+#endif
+
+    va_end(args);
+  }
 
   exit(code);
+
 }
 
