@@ -13,15 +13,20 @@
  *                MdcPrintBoxLine()       - Print horizontal line of a box *
  *                MdcPrintYesNo()         - Print Yes, No or Unknown       *
  *                MdcPrintImageLayout()   - Print layout of a raw image    *
+ *                MdcPrintValue()         - Print numeric value            *
  *                MdcLowStr()             - Make string lower case         *
+ *                MdcUpStr()              - Make string upper case         *
  *                MdcKillSpaces()         - Remove first/last spaces       *
+ *                MdcRemoveAllSpaces()    - Remove all spaces from string  *
  *                MdcRemoveEnter()        - Remove <enter> from string     *
- *                MdcGetStrInput()        - Get string & remove <enter>    *
+ *                MdcGetStrLine()         - Get string skipping comment    *
+ *                MdcGetStrInput()        - Get string from input with '\n'*
+ *                MdcGetSubStr()          - Get substr between separators  *
  *                MdcGetSafeString()      - Copy & add terminating char    *
  *                MdcPutDefault()         - Get (default) answer           *
+ *                MdcGetRange()           - Get a range from the a list    *
  *                MdcHandleEcatList()     - Get a list in ecat style       *
  *                MdcHandleNormList()     - Get a list in normal style     *
- *                MdcGetPixelRange()      - Get the col/row ranges of list *
  *                MdcHandlePixelList()    - Get a list of pixels           *
  *                MdcGetStrAcquisition()  - Get string for acquisition type*
  *                MdcGetStrRawConv()      - Get string of raw type         *
@@ -30,13 +35,18 @@
  *                MdcGetStrPixelType()    - Get string of pixel type       *
  *                MdcGetStrColorMap()     - Get string of colormap         *
  *                MdcGetStrYesNo()        - Get string "yes" or "no"       *
+ *                MdcGetStrSlProjection() - Get string slice projection    *
  *                MdcGetStrPatSlOrient()  - Get string patient/slice orient*
- *                MdcGetStrPatientPos()   - Get string patient position    *
- *                MdcGetStrPatientOrient()- Get string patient orientation *
- *                MdcGetStrPatientRot()   - Get string patient rotation    * 
+ *                MdcGetStrPatPos()       - Get string patient position    *
+ *                MdcGetStrPatOrient()    - Get string patient orientation *
  *                MdcGetStrSliceOrient()  - Get string slice   orientation *
  *                MdcGetStrRotation()     - Get string rotation direction  *
  *                MdcGetStrMotion()       - Get string detector motion     *
+ *                MdcGetStrModality()     - Get string modality            *
+ *                MdcGetStrGSpectNesting()- Get string GSPECT nesting      *
+ *                MdcGetStrHHMMSS()       - Get string hrs:mins:secs       *
+ *                MdcGetIntModality()     - Get int modality type          *
+ *                MdcGetIntSliceOrient()  - Get int slice orientation      *
  *                MdcGetLibLongVersion()  - Get string of library version  *
  *                MdcGetLibShortVersion() - Get string of short   version  *
  *                MdcCheckStrSize()       - Check if we can add a string   *
@@ -50,11 +60,11 @@
  *                MdcAddCompressionExt()  - Add  compression extension     *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-fancy.c,v 1.1.1.1 2000/10/28 16:51:00 enlf Exp $
+/* $Id: m-fancy.c,v 1.51 2004/01/04 22:36:16 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -74,6 +84,23 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#include <ctype.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
+
+
 #include "m-fancy.h"
 
 /****************************************************************************
@@ -84,28 +111,28 @@ void MdcPrintLine(char c, int length)
 {
   int i;
 
-  for (i=0; i<length; i++) printf("%c",c);
-  printf("\n");
+  for (i=0; i<length; i++) MdcPrntScrn("%c",c);
+  MdcPrntScrn("\n");
 
 }
 
 void MdcPrintChar(int c)
 {
-  if (c == '\0') printf("<null>");
+  if (c == '\0') MdcPrntScrn("<null>");
   else if (c==9 || c==13 || c==10) putchar(c);
   else if (c >= 32) putchar(c);
-  else if (c==EOF) printf("<EOF>");
-  else printf("<%u>",c);
+  else if (c==EOF) MdcPrntScrn("<EOF>");
+  else MdcPrntScrn("<%u>",c);
 }
 
 void MdcPrintStr(char *str)
 {
   int t=strlen(str);
 
-  if ( t == 0 ) printf("<null>");
-  else printf("%s",str);
+  if ( t == 0 ) MdcPrntScrn("<null>");
+  else MdcPrntScrn("%s",str);
 
-  printf("\n");
+  MdcPrntScrn("\n");
 
 }
 
@@ -113,18 +140,18 @@ void MdcPrintBoxLine(char c, int t)
 {
   int i;
 
-  printf("\t\t#");
-  for (i=-1;i<=t;i++) printf("%c",c);
-  printf("#\n");
+  MdcPrntScrn("\t\t#");
+  for (i=-1;i<=t;i++) MdcPrntScrn("%c",c);
+  MdcPrntScrn("#\n");
   
 }
 
 void MdcPrintYesNo(int value )
 {
   switch ( value ) {
-    case MDC_NO : printf("(= No)\n");      break;
-    case MDC_YES: printf("(= Yes)\n");     break;
-    default     : printf("(= Unknown)\n"); break;
+    case MDC_NO : MdcPrntScrn("(= No)\n");      break;
+    case MDC_YES: MdcPrntScrn("(= Yes)\n");     break;
+    default     : MdcPrntScrn("(= Unknown)\n"); break;
   }
 }
 
@@ -135,33 +162,104 @@ void MdcPrintImageLayout(FILEINFO *fi, Uint32 gen, Uint32 img
   Uint32 i;
 
   MdcPrintLine('-',MDC_FULL_LENGTH);
-  printf("\t\t\tSUMMARY OF IMAGE LAYOUT\n");
+  MdcPrntScrn("\t\t\tSUMMARY OF IMAGE LAYOUT\n");
   MdcPrintLine('-',MDC_FULL_LENGTH);
 
   if ((gen==0) && (img==0))  MdcPrintBoxLine('-',MDC_BOX_SIZE);
   if (gen!=0) {
     MdcPrintBoxLine('-',MDC_BOX_SIZE);
-    printf("\t\t| General Header   |  \t(%u)\n",gen);
+    MdcPrntScrn("\t\t| General Header   |  \t(%u)\n",gen);
     MdcPrintBoxLine('-',MDC_BOX_SIZE);
   }
   for (i=0; i<fi->number; i++) {
      id = &fi->image[i];
      if ( ((i==0) && (img>0)) || (repeat) ) {
        if ( ! ((i==0) && (gen>0)) )MdcPrintBoxLine('-',MDC_BOX_SIZE); 
-       printf("\t\t| Image   Header   |  \t(%u)\n",img);
+       MdcPrntScrn("\t\t| Image   Header   |  \t(%u)\n",img);
        MdcPrintBoxLine('-',MDC_BOX_SIZE);
      }
-     printf("\t\t! Image #%-4u ",i+1);
-     if (fi->endian != MDC_HOST_ENDIAN) printf("swap !"); 
-     else printf("     !");
-     printf("\t(%ux%ux%u)",id->width,id->height,MdcType2Bytes(id->type));
-     if (abs!=NULL) if (abs[i] > 0) printf("\tOFFSET: %u",abs[i]);
-     printf("\n");
+     MdcPrntScrn("\t\t! Image #%-4u ",i+1);
+     if (fi->endian != MDC_HOST_ENDIAN) MdcPrntScrn("swap !"); 
+     else MdcPrntScrn("     !");
+     MdcPrntScrn("\t(%ux%ux%u)",id->width,id->height,MdcType2Bytes(id->type));
+     if (abs!=NULL) if (abs[i] > 0) MdcPrntScrn("\tOFFSET: %u",abs[i]);
+     MdcPrntScrn("\n");
 
   }
   MdcPrintBoxLine('-',MDC_BOX_SIZE);
 }
 
+int MdcPrintValue(FILE *fp, Uint8 *pvalue, Uint16 type)
+{
+  
+  switch (type) {
+   case BIT8_S:
+    {
+      Int8 *val = (Int8 *) pvalue;
+      fprintf(fp,"%hd",val[0]);
+    }
+    break;
+   case BIT8_U:
+    {
+      Uint8 *val = (Uint8 *) pvalue;
+      fprintf(fp,"%hu",val[0]);
+    }
+    break;
+   case BIT16_S:
+    {
+      Int16 *val = (Int16 *) pvalue;
+      fprintf(fp,"%hd",val[0]);
+    }
+    break;
+   case BIT16_U:
+    {
+      Uint16 *val = (Uint16 *) pvalue;
+      fprintf(fp,"%hu",val[0]);
+    }
+    break;
+   case BIT32_S:
+    {
+      Int32 *val = (Int32 *) pvalue;
+      fprintf(fp,"%d",val[0]);
+    }
+    break;
+   case BIT32_U:
+    {
+      Uint32 *val = (Uint32 *) pvalue;
+      fprintf(fp,"%d",val[0]);
+    }
+    break;
+#ifdef HAVE_8BYTE_INT
+   case BIT64_S:
+    {
+      Int64 *val = (Int64 *) pvalue;
+      fprintf(fp,"%ld",val[0]);
+    }
+    break;
+   case BIT64_U:
+    {
+      Uint64 *val = (Uint64 *) pvalue;
+      fprintf(fp,"%lu",val[0]);
+    }
+    break;
+#endif
+   case FLT32:
+    {
+      float *val = (float *) pvalue;
+      fprintf(fp,"%+e",val[0]);
+    }
+    break;
+   case FLT64:
+    {
+      double *val = (double *) pvalue;
+      fprintf(fp,"%+e",val[0]);
+    }
+    break;
+  }
+
+  return(ferror(fp));
+}
+
 void MdcLowStr(char *str)
 { 
   char *c;
@@ -172,6 +270,15 @@ void MdcLowStr(char *str)
 
 }
 
+void MdcUpStr(char *str)
+{
+  char *c;
+
+  c=str;
+
+  while(*c) { *c=toupper((int)*c); c++; }
+
+}
 
 void MdcKillSpaces(char string[]) /* kill first and last spaces */
 {
@@ -201,6 +308,25 @@ void MdcKillSpaces(char string[]) /* kill first and last spaces */
   } 
 }
 
+void MdcRemoveAllSpaces(char string[]) /* remove all spaces */
+{
+  int i=0, j=0, length;
+
+  length = strlen(string);
+
+  while (i < length) {
+       if (isspace((int)string[i])) {
+         i+=1;
+       }else{
+         string[j++] = string[i++];
+       }
+  }
+
+  string[j]='\0';
+
+}
+
+
 void MdcRemoveEnter(char string[])
 {
   char *p;
@@ -210,10 +336,52 @@ void MdcRemoveEnter(char string[])
 
 } 
 
-void MdcGetStrInput(char string[], Uint32 maxchars)
+void MdcGetStrLine(char string[], int maxchars, FILE *fp)
+{
+  /* skip comment lines beginning with '#' */
+  do {
+    if (fgets(string,maxchars,fp) == NULL) return;
+  }while (string[0] == '#');
+}
+
+void MdcGetStrInput(char string[], int maxchars)
 { 
-  fscanf(stdin,"%s",string); 
-  string[maxchars-1]='\0';         /* make sure it's null terminated */
+  MdcGetStrLine(string,maxchars,stdin);
+}
+
+int MdcGetSubStr(char *dest, char *src, int dmax, char sep, int n)
+{
+  Uint32 i, b, cnt=1, length, sublength=0;
+
+  length = strlen(src);
+
+  if (length == 0) return(MDC_NO);
+
+  /* get begin substr */
+  for (b=0; b<length; b++) {
+     if (src[b] == sep) cnt+=1;
+     if (cnt == n) break;
+  }
+
+  /* n-th substr not found */
+  if (cnt != n) return(MDC_NO);
+
+  /* get length substr */
+  b+=1;
+  for (i=b; i<length; i++) {
+     if (src[i] == sep) break;
+     sublength+=1;
+  }
+
+  if ((sublength == 0) || (sublength >= dmax)) return(MDC_NO);
+
+  strncpy(dest,&src[b],sublength);
+
+  dest[sublength] = '\0'; 
+
+  MdcKillSpaces(dest);
+
+  return(MDC_YES);
 }
  
 void MdcGetSafeString(char *dest, char *src, Uint32 length, Uint32 maximum)
@@ -227,241 +395,292 @@ void MdcGetSafeString(char *dest, char *src, Uint32 length, Uint32 maximum)
      memcpy(dest,src,MAX);
      dest[MAX]='\0';
    }
-} 
+}
+
+int MdcUseDefault(char string[])
+{
+  /* <enter> = default */
+  if (string[0] == '\n' || string[0] == '\r') return(1);
+  return(0);
+}
                     /* string[] = MDC_2KB_OFFSET               */
 int MdcPutDefault(char string[]) /* 1=default  or 0=no default */
 {
-  fgets(string,MDC_2KB_OFFSET-1,stdin);
-  if (string[0] == '\n' || string[0] == '\r') return(1);
+  MdcGetStrLine(string,MDC_2KB_OFFSET-1,stdin);
+  if (MdcUseDefault(string)) return(1);
   MdcKillSpaces(string);
   return(0);
 
 }
 
-int MdcHandleEcatList(char *s, Uint32 **item, Uint32 max)
+int MdcGetRange(char *item, Uint32 *from, Uint32 *to, Uint32 *step)
 {
+  Uint32 a1, a2, t;
 
-  Uint32 a1, a2, tmp, i;
-
-  while (1) {
-
-    if (atol(s) < 0 ) { break; } 
-
-    if ( strstr(s,"...") != 0 ) { /* a range ... */
-      sscanf(s,"%u...%u",&a1,&a2);
-      if (a1 > max) {
-        MdcPrntWarn("Number %u too big!",a1);
-        MdcFlushInput(); return(MDC_BAD_CODE);
-      }
-      if (a2 > max) {
-        MdcPrntWarn("Number %u too big!",a2);
-        MdcFlushInput(); return(MDC_BAD_CODE);
-      } 
-      if ( (a1==0) || (a2==0) ) {
-        for (i=1; i<=max; i++) (*item)[i]=MDC_YES;
-        (*item)[0]=max;
-        break;
-      }
-      if (a1 > a2) {
-        tmp = a1;
-        a1  = a2;
-        a2  = tmp;
-      }
-      for (i=a1; i<=a2; i++) {
-         if ((*item)[i] == MDC_NO) {
-           (*item)[i] = MDC_YES; 
-           (*item)[0] += 1;
-         }
-      }
-    }else{ /* a single ... */
-      sscanf(s,"%u",&a1);
-      if (a1 > max) {
-        MdcPrntWarn("Number %u too big!",a1);
-        MdcFlushInput(); return(MDC_BAD_CODE);
-      }
-
-      if (a1 == 0) {
-        for (i=1; i<=max; i++) (*item)[i]=MDC_YES;
-        (*item)[0]=max;
-        break;
-      }
-
-      if ((*item)[a1] == MDC_NO) {
-        (*item)[a1] = MDC_YES;
-        (*item)[0] += 1;
-      }
-    }
-
-    MdcGetStrInput(s,MDC_2KB_OFFSET);
-
+  /* read range values */
+  if (strchr(item,':') != 0 ) {
+    /* interval */
+    sscanf(item,"%u:%u:%u",&a1,&t,&a2);
+  }else if ( strstr(item,"...") != 0 ) {
+    /* range v1 */
+    sscanf(item,"%u...%u",&a1,&a2); t=1;
+  }else if ( strstr(item,"-") != 0 ) {
+    /* range v2 */
+    sscanf(item,"%u-%u",&a1,&a2); t=1;
+  }else{
+    /* single */
+    sscanf(item,"%u",&a1); a2=a1;   t=1;
   }
 
-  MdcFlushInput(); return(MDC_OK);
+  /* some sanity checks */ 
+  if (t == 0) t = 1;
 
-}
-
-int MdcHandleNormList(char *s,Uint32 **inrs,Uint32 *it,Uint32 *bt,Uint32 max)
-{
-  Uint32 a1, a2, i;
-
-  while(1) {
- 
-    if (atol(s) < 0 ) { break; }
-
-    if ( strstr(s,"...") != 0 ) { /* a range ... */
-      sscanf(s,"%u...%u",&a1,&a2);
-      if (a1 > max) {
-        MdcPrntWarn("Number %u too big!",a1);
-        break;
-      }
-      if (a2 > max) {
-        MdcPrntWarn("Number %u too big!",a2);
-        break;
-      }
-      if ( (a1==0) || (a2==0) ) {
-        (*inrs)[1] = 0;
-        *it = 2; 
-        break;
-      }
-
-      if (a1 > a2) {
-        for (i=a1; i>=a2; i--) {
-          (*inrs)[*it] = i;
-          *it += 1;
-          if ( (*it % MDC_BUF_ITMS) == 0 ) {
-            if (((*inrs)=(Uint32 *)MdcRealloc((*inrs),
-                         (*bt)*MDC_BUF_ITMS*sizeof(Uint32)))==NULL){
-              MdcPrntWarn("Couldn't realloc images number buffer");
-              MdcFlushInput(); return(MDC_BAD_ALLOC);
-            }
-            *bt += 1;
-          }
-        }
-      }else {
-        for (i=a1; i<=a2; i++) {
-          (*inrs)[*it] = i;
-          *it += 1;
-          if ( (*it % MDC_BUF_ITMS) == 0 ) {
-            if (((*inrs)=(Uint32 *)MdcRealloc((*inrs),
-                         (*bt)*MDC_BUF_ITMS*sizeof(Uint32)))==NULL){
-              MdcPrntWarn("Couldn't realloc images number buffer");
-              MdcFlushInput(); return(MDC_BAD_ALLOC);
-            }
-            *bt += 1;
-          }
-        }
-      }
-
-    }else{ /* a single ... */
-      sscanf(s,"%u",&a1);
-      if (a1 > max) {
-        MdcPrntWarn("Number %u too big!",a1);
-        break;
-      }
-      if ( a1 == 0 ) {
-        (*inrs)[1] = 0;
-        *it = 2;
-        break;
-      }
-      (*inrs)[*it] = a1;
-      (*it) += 1;
-      if ( (*it % MDC_BUF_ITMS) == 0 ) {
-        if (((*inrs)=(Uint32 *)MdcRealloc((*inrs),
-                     (*bt)*MDC_BUF_ITMS*sizeof(Uint32)))==NULL){
-          MdcPrntWarn("Couldn't realloc images number buffer");
-          MdcFlushInput(); return(MDC_BAD_ALLOC);
-        }
-        *bt += 1;
-      }
-    }
-
-    MdcGetStrInput(s,MDC_2KB_OFFSET);
-
-  }
-
-  MdcFlushInput(); return(MDC_OK);
-
-}
-
-int MdcGetPixelRange(char *string, Uint32 *from, Uint32 *to)
-{
-  Uint32 a1, a2, tmp;
-
-  if ( strstr(string,"...") != 0 ) { /* a range ... */
-    sscanf(string,"%d...%d",&a1,&a2);
-    if ( (a1==0) || (a2==0) ) {
-      *from = 0; *to = 0;
-    }else{
-     if (a1 > a2) {
-       tmp = a1;
-       a1  = a2;
-       a2  = tmp;
-     }
-     *from = a1; *to = a2;
-    }
-  }else{ /* a single */
-    sscanf(string,"%u",&a1);
-    if (a1 == 0) {
-      *from=0; *to=0;
-    }else{
-      *from=a1; *to=a1;
-    }
-  }
+  *from = a1; *to = a2; *step = t;
 
   return(MDC_OK);
 }
 
-int MdcHandlePixelList(char *list, Uint32 **cols, Uint32 **rows, 
-                    Uint32 *it, Uint32 *bt)
+char *MdcHandleEcatList(char *list, Uint32 **dims, Uint32 max)
 {
-   int error;
-   Uint32 r_from, r_to, c_from, c_to, r, c;
-   char *col, *row;
-   
-   while (1) {
-     if (atol(list) < 0 ) { MdcFlushInput(); return(MDC_OK); }
-     col=list;
-     row=strchr(list,',');
-     if ( row == NULL) {
-       MdcPrntWarn("Wrong input!\n");
-       MdcFlushInput(); return(MDC_BAD_CODE);
-     }else{
-       *row = '\0';
-       row += 1;
+  int ITEM_FOUND=MDC_NO, REVERSED, HANDLE;
+  Uint32 a1, a2, t, i, l, length;
+  char *p, *item;
+
+  length = strlen(list);
+
+  /* <enter> default = all */
+  if (MdcUseDefault(list)) {
+    for (i=1; i<=max; i++) (*dims)[i]=MDC_YES;
+    (*dims)[0]=max;
+    return(NULL);
+  }
+
+  /* loop through string with entire list */
+  for (p=list, item=list, l=0; l<=length; l++) {
+
+     /* separate items: begins at digit, ends at space (or \t, \n, ...) */
+
+     if (ITEM_FOUND == MDC_NO) {
+
+       if (isdigit((int)p[l])) { item=&p[l]; ITEM_FOUND=MDC_YES; }
+
+     }else if (isspace((int)p[l]) || p[l]=='\0') {
+
+       p[l]='\0';
+
+       if (MdcGetRange(item,&a1,&a2,&t) != MDC_OK)
+         return("Error reading range item");
+
+       if (a1 > max) a1 = max;
+       if (a2 > max) a2 = max;
+
+       if ( (a1==0) || (a2==0) ) {
+         for (i=1; i<=max; i++) (*dims)[i]=MDC_YES;
+         (*dims)[0]=max;
+         break;
+       }
+
+       /* reversed range ? */
+       REVERSED = (a1 > a2) ? MDC_YES : MDC_NO;
+
+       /* initialize and get image numbers */
+       i = a1; HANDLE = MDC_YES;
+       do {
+
+         /* include image number */
+         if ((*dims)[i] == MDC_NO) {
+           (*dims)[i] = MDC_YES; 
+           (*dims)[0] += 1;
+         }
+
+         if ((REVERSED == MDC_YES) && (i < t)) break;
+          
+         /* set next image number in range */
+         i = (REVERSED == MDC_YES) ? (i-t) : (i+t);
+
+         /* check end of range */
+         if (REVERSED == MDC_YES) {
+           if (i < a2) HANDLE = MDC_NO;
+         }else{
+           if (i > a2) HANDLE = MDC_NO;
+         }
+
+       }while(HANDLE == MDC_YES);
+
+       ITEM_FOUND = MDC_NO;
+
+     } 
+  }
+
+  return(NULL);
+
+}
+
+char *MdcHandleNormList(char *list,Uint32 **inrs,Uint32 *it
+                                  ,Uint32 *bt,Uint32 max)
+{
+  int ITEM_FOUND=MDC_NO, HANDLE, REVERSED;
+  Uint32 a1, a2, t, i, l, length;
+  char *p, *item;
+
+  length = strlen(list);
+
+  /* <enter> = default: all */
+  if (MdcUseDefault(list)) { 
+    (*inrs)[1] = 0; *it = 2; return(NULL);
+  }
+
+  /* loop through string with entire list */
+  for (p=list, item=list, l=0; l<=length; l++) {
+
+     /* separate items: begins at digit, ends at space (or \t, \n, ...) */
+
+     if (ITEM_FOUND == MDC_NO) {
+
+       if (isdigit((int)p[l])) { item=&p[l]; ITEM_FOUND=MDC_YES; }
+
+     }else if (isspace((int)p[l]) || p[l]=='\0') {
+
+       p[l]='\0';
+
+       if (MdcGetRange(item,&a1,&a2,&t) != MDC_OK)
+         return("Error reading range item");
+
+       if (a1 > max) a1 = max;
+       if (a2 > max) a2 = max;
+
+       if ( (a1==0) || (a2==0) ) {
+         (*inrs)[1] = 0;
+         *it = 2;
+         return(NULL);
+       }
+
+       /* reversed range ? */
+       REVERSED = (a1 > a2) ? MDC_YES : MDC_NO;
+
+       /* initialize and get image numbers */
+       i = a1; HANDLE = MDC_YES;
+       do {
+
+         /* store image number */
+         (*inrs)[*it] = i;
+         *it += 1;
+         if ( (*it % MDC_BUF_ITMS) == 0 ) {
+           if (((*inrs)=(Uint32 *)MdcRealloc((*inrs),
+                        (*bt)*MDC_BUF_ITMS*sizeof(Uint32)))==NULL){
+             return("Couldn't realloc images number buffer");
+           }
+           *bt += 1;
+         }
+
+         if ((REVERSED == MDC_YES) && (i < t)) break;
+
+         /* set next image number in range */
+         i = (REVERSED == MDC_YES) ? (i-t) : (i+t);
+
+         /* check end of range */
+         if (REVERSED == MDC_YES) {
+           if (i < a2) HANDLE = MDC_NO;
+         }else{
+           if (i > a2) HANDLE = MDC_NO;
+         }
+         
+       }while (HANDLE == MDC_YES);
+
+       ITEM_FOUND = MDC_NO;
+
      }
 
-     if ((error=MdcGetPixelRange(col,&c_from,&c_to)) != MDC_OK) {
-       MdcFlushInput(); return(error);
-     }
-     if ((error=MdcGetPixelRange(row,&r_from,&r_to)) != MDC_OK) {
-       MdcFlushInput(); return(error);
-     }
+  }
+
+  return(NULL);
+
+}
+
+char *MdcHandlePixelList(char *list, Uint32 **cols, Uint32 **rows, 
+                         Uint32 *it, Uint32 *bt)
+{
+  int ITEM_FOUND=MDC_NO;
+  Uint32 r_from, r_to, r_step, c_from, c_to, c_step;
+  Uint32 r, c, l, length, tmp;
+  char *col, *row;
+  char *p, *item;
+
+  length = strlen(list);
+
+  /* <enter> default = all */
+  if (MdcUseDefault(list)) {
+    (*cols)[*it] = 0;
+    (*rows)[*it] = 0;
+    *it+=1;
+    return(NULL);
+  }
+
+  /* loop through string with entire list */
+  for (p=list, item=list, l=0; l<=length; l++) {
+
+     /* separate items: begins at digit, ends at space (or \t, \n, ...) */
+
+     if (ITEM_FOUND == MDC_NO) {
+
+       if (isdigit((int)p[l])) { item=&p[l]; ITEM_FOUND=MDC_YES; }
+
+     }else if (isspace((int)p[l]) || p[l]=='\0') {
+
+       p[l]='\0';
+
+       col=item;
+       row=strchr(item,',');
+
+       if ( row == NULL) return("Wrong input!"); 
+
+       *row = '\0'; row += 1;
+
+       if (MdcGetRange(col,&c_from,&c_to,&c_step) != MDC_OK)
+         return("Error reading column range");
+
+       /* some checks */
+       if (c_from == 0 || c_to == 0) {
+         c_from = 0; c_to = 0;
+       }else if (c_from > c_to) {
+         tmp = c_from; c_from = c_to; c_to = tmp;
+       } 
+
+       if (MdcGetRange(row,&r_from,&r_to,&r_step) != MDC_OK)
+         return("Error reading row range"); 
+
+       /* some checks */
+       if (r_from == 0 || r_to == 0) {
+         r_from = 0; r_to = 0;
+       }else if (r_from > r_to) {
+         tmp = r_from; r_from = r_to; r_to = tmp;
+       } 
        
-     for (r=r_from; r<=r_to; r++)
-     for (c=c_from; c<=c_to; c++) {
-        (*cols)[*it] = c;
-        (*rows)[*it] = r;
-        *it+=1;
-        if ( (*it % MDC_BUF_ITMS) == 0 ) {
-          if ( ((*cols)=(Uint32 *)MdcRealloc((*cols),
-                      (*bt)*MDC_BUF_ITMS*sizeof(Uint32)))==NULL){
-            MdcPrntWarn("Couldn't realloc pixels column buffer");
-            MdcFlushInput(); return(MDC_BAD_ALLOC);
-          }
-          if (((*rows)=(Uint32 *)MdcRealloc((*rows),
-                      (*bt)*MDC_BUF_ITMS*sizeof(Uint32)))==NULL){
-            MdcPrntWarn("Couldn't realloc pixels row buffer");
-            MdcFlushInput(); return(MDC_BAD_ALLOC);
-          }
+       for (r=r_from; r<=r_to; r+=r_step)
+       for (c=c_from; c<=c_to; c+=c_step) {
+          (*cols)[*it] = c;
+          (*rows)[*it] = r;
+          *it+=1;
+          if ( (*it % MDC_BUF_ITMS) == 0 ) {
+            if ( ((*cols)=(Uint32 *)MdcRealloc((*cols),
+                        (*bt)*MDC_BUF_ITMS*sizeof(Uint32))) == NULL) {
+              return("Couldn't realloc pixels column buffer");
+            }
+            if (((*rows)=(Uint32 *)MdcRealloc((*rows),
+                        (*bt)*MDC_BUF_ITMS*sizeof(Uint32))) == NULL) {
+              return("Couldn't realloc pixels row buffer");
+	    }
+	  }
           *bt+=1;
-        }
+       }
+
+       ITEM_FOUND = MDC_NO;
+
      }
+  }
 
-     MdcGetStrInput(list,MDC_2KB_OFFSET);
+  return(NULL);
 
-   }
-
-   MdcFlushInput(); return(MDC_OK);
 }
 
 char *MdcGetStrAcquisition(int acq_type)
@@ -473,6 +692,10 @@ char *MdcGetStrAcquisition(int acq_type)
                                        break;
     case MDC_ACQUISITION_TOMO   : return("Tomographic");
                                        break;
+    case MDC_ACQUISITION_GATED  : return("Gated");
+                                       break;
+    case MDC_ACQUISITION_GSPECT : return("GSPECT");
+                                       break;
     default                     : return("Unknown");
   }
 } 
@@ -520,8 +743,9 @@ char *MdcGetStrPixelType(int type)
    case  BIT64_U: return("Uint64"); break;
    case    FLT32: return("IEEE float"); break;
    case    FLT64: return("IEEE double"); break;
-   case  VAXFL32: return("VAX  float"); break;
    case    ASCII: return("ASCII"); break;
+   case  VAXFL32: return("VAX  float"); break;
+   case   COLRGB: return("RGB24 triplets"); break;
    default      : return("Unknown");
   }
 }
@@ -529,16 +753,13 @@ char *MdcGetStrPixelType(int type)
 char *MdcGetStrColorMap(int map)
 {
   switch (map) {
-   case MDC_MAP_PRESENT : return("present"); break;
-   case MDC_MAP_GRAY    : if (MDC_CMAP_INVERS == MDC_NO)
-                             return("gray normal");
-                           else
-                             return("gray invers");
-                          break;
-   case MDC_MAP_RAINBOW : return("rainbow"); break;
-   case MDC_MAP_COMBINED: return("combined"); break; 
-   case MDC_MAP_HOTMETAL: return("hotmetal"); break;
-   case MDC_MAP_LOADED  : return("loaded LUT"); break;
+   case MDC_MAP_PRESENT : return("present");     break;
+   case MDC_MAP_GRAY    : return("gray normal"); break;
+   case MDC_MAP_INVERTED: return("gray invers"); break;
+   case MDC_MAP_RAINBOW : return("rainbow");     break;
+   case MDC_MAP_COMBINED: return("combined");    break; 
+   case MDC_MAP_HOTMETAL: return("hotmetal");    break;
+   case MDC_MAP_LOADED  : return("loaded LUT");  break;
    default              : return("Unknown");
   }
 }
@@ -552,29 +773,45 @@ char *MdcGetStrYesNo(int boolean)
   }
 }
 
+char *MdcGetStrSlProjection(int slice_projection)
+{
+  switch (slice_projection) {
+
+    case MDC_TRANSAXIAL: strcpy(mdcbufr,"XY - Transaxial");
+        break;
+    case MDC_SAGITTAL  : strcpy(mdcbufr,"YZ - Sagittal");
+        break;
+    case MDC_CORONAL   : strcpy(mdcbufr,"XZ - Coronal");
+        break;
+    default: strcpy(mdcbufr,"Unknown");
+  }
+
+  return(mdcbufr);
+}
+
 char *MdcGetStrPatSlOrient(int patient_slice_orient)
 {
   switch (patient_slice_orient) {
 
-   case MDC_SUPINE_HEADFIRST_TRANSVERSAL:
+   case MDC_SUPINE_HEADFIRST_TRANSAXIAL:
     strcpy(mdcbufr,"Supine;HeadFirst;Transversal"); break;
    case MDC_SUPINE_HEADFIRST_SAGITTAL   : 
     strcpy(mdcbufr,"Supine;HeadFirst;Sagittal");    break;
    case MDC_SUPINE_HEADFIRST_CORONAL    : 
     strcpy(mdcbufr,"Supine;HeadFirst;Coronal");     break;
-   case MDC_SUPINE_FEETFIRST_TRANSVERSAL: 
+   case MDC_SUPINE_FEETFIRST_TRANSAXIAL: 
     strcpy(mdcbufr,"Supine;FeetFirst;Transversal"); break;
    case MDC_SUPINE_FEETFIRST_SAGITTAL   : 
     strcpy(mdcbufr,"Supine;FeetFirst;Sagittal");    break;
    case MDC_SUPINE_FEETFIRST_CORONAL    : 
     strcpy(mdcbufr,"Supine;FeetFirst;Coronal");     break;
-   case MDC_PRONE_HEADFIRST_TRANSVERSAL : 
+   case MDC_PRONE_HEADFIRST_TRANSAXIAL : 
     strcpy(mdcbufr,"Prone;HeadFirst;Transversal");  break;
    case MDC_PRONE_HEADFIRST_SAGITTAL    : 
     strcpy(mdcbufr,"Prone;HeadFirst;Sagittal");     break;
    case MDC_PRONE_HEADFIRST_CORONAL     : 
     strcpy(mdcbufr,"Prone;HeadFirst;Coronal");      break;
-   case MDC_PRONE_FEETFIRST_TRANSVERSAL : 
+   case MDC_PRONE_FEETFIRST_TRANSAXIAL : 
     strcpy(mdcbufr,"Prone;FeetFirst;Transversal");   break;
    case MDC_PRONE_FEETFIRST_SAGITTAL    : 
     strcpy(mdcbufr,"Prone;FeetFirst;Sagittal");     break;
@@ -588,46 +825,46 @@ char *MdcGetStrPatSlOrient(int patient_slice_orient)
   return(mdcbufr);
 } 
 
-char *MdcGetStrPatientPos(int patient_slice_orient)
+char *MdcGetStrPatPos(int patient_slice_orient)
 {
   switch (patient_slice_orient) {
-
-   case MDC_SUPINE_HEADFIRST_TRANSVERSAL: 
-   case MDC_SUPINE_HEADFIRST_SAGITTAL   :
-   case MDC_SUPINE_HEADFIRST_CORONAL    :
-   case MDC_SUPINE_FEETFIRST_TRANSVERSAL:
-   case MDC_SUPINE_FEETFIRST_SAGITTAL   :
-   case MDC_SUPINE_FEETFIRST_CORONAL    :
-    strcpy(mdcbufr,"supine"); break;
-   case MDC_PRONE_HEADFIRST_TRANSVERSAL :
-   case MDC_PRONE_HEADFIRST_SAGITTAL    :
-   case MDC_PRONE_HEADFIRST_CORONAL     :
-   case MDC_PRONE_FEETFIRST_TRANSVERSAL :
-   case MDC_PRONE_FEETFIRST_SAGITTAL    :
-   case MDC_PRONE_FEETFIRST_CORONAL     :
-    strcpy(mdcbufr,"prone"); break;
-   default                              :
-    strcpy(mdcbufr,"Unknown");
-
+    case MDC_SUPINE_HEADFIRST_TRANSAXIAL: 
+    case MDC_SUPINE_HEADFIRST_SAGITTAL   :
+    case MDC_SUPINE_HEADFIRST_CORONAL    :
+        strcpy(mdcbufr,"HFS"); break;
+    case MDC_SUPINE_FEETFIRST_TRANSAXIAL:
+    case MDC_SUPINE_FEETFIRST_SAGITTAL   :
+    case MDC_SUPINE_FEETFIRST_CORONAL    :
+        strcpy(mdcbufr,"FFS"); break;
+    case MDC_PRONE_HEADFIRST_TRANSAXIAL :
+    case MDC_PRONE_HEADFIRST_SAGITTAL    :
+    case MDC_PRONE_HEADFIRST_CORONAL     :
+        strcpy(mdcbufr,"HFP"); break;
+    case MDC_PRONE_FEETFIRST_TRANSAXIAL :
+    case MDC_PRONE_FEETFIRST_SAGITTAL    :
+    case MDC_PRONE_FEETFIRST_CORONAL     :
+        strcpy(mdcbufr,"FFP"); break;
+    default                              :
+        strcpy(mdcbufr,"Unknown");
   }
 
   return(mdcbufr);
 
 }
 
-char *MdcGetStrPatientOrient(int patient_slice_orient)
+char *MdcGetStrPatOrient(int patient_slice_orient)
 {
   switch (patient_slice_orient) {
-     case MDC_SUPINE_HEADFIRST_TRANSVERSAL: strcpy(mdcbufr,"L\\P"); break;
+     case MDC_SUPINE_HEADFIRST_TRANSAXIAL: strcpy(mdcbufr,"L\\P"); break;
      case MDC_SUPINE_HEADFIRST_SAGITTAL   : strcpy(mdcbufr,"P\\F"); break;
      case MDC_SUPINE_HEADFIRST_CORONAL    : strcpy(mdcbufr,"L\\F"); break;
-     case MDC_SUPINE_FEETFIRST_TRANSVERSAL: strcpy(mdcbufr,"R\\P"); break;
+     case MDC_SUPINE_FEETFIRST_TRANSAXIAL: strcpy(mdcbufr,"R\\P"); break;
      case MDC_SUPINE_FEETFIRST_SAGITTAL   : strcpy(mdcbufr,"P\\H"); break;
      case MDC_SUPINE_FEETFIRST_CORONAL    : strcpy(mdcbufr,"R\\H"); break;
-     case MDC_PRONE_HEADFIRST_TRANSVERSAL : strcpy(mdcbufr,"R\\A"); break;
+     case MDC_PRONE_HEADFIRST_TRANSAXIAL : strcpy(mdcbufr,"R\\A"); break;
      case MDC_PRONE_HEADFIRST_SAGITTAL    : strcpy(mdcbufr,"A\\F"); break;
      case MDC_PRONE_HEADFIRST_CORONAL     : strcpy(mdcbufr,"R\\F"); break;
-     case MDC_PRONE_FEETFIRST_TRANSVERSAL : strcpy(mdcbufr,"L\\A"); break;
+     case MDC_PRONE_FEETFIRST_TRANSAXIAL : strcpy(mdcbufr,"L\\A"); break;
      case MDC_PRONE_FEETFIRST_SAGITTAL    : strcpy(mdcbufr,"A\\H"); break;
      case MDC_PRONE_FEETFIRST_CORONAL     : strcpy(mdcbufr,"L\\H"); break;
      default                              : strcpy(mdcbufr,"Unknown");
@@ -636,38 +873,13 @@ char *MdcGetStrPatientOrient(int patient_slice_orient)
   return(mdcbufr);
 }
 
-char *MdcGetStrPatientRot(int patient_slice_orient)
-{
-
-  switch (patient_slice_orient) {
-     case MDC_SUPINE_HEADFIRST_TRANSVERSAL:
-     case MDC_SUPINE_HEADFIRST_SAGITTAL   :
-     case MDC_SUPINE_HEADFIRST_CORONAL    :
-     case MDC_PRONE_HEADFIRST_TRANSVERSAL :
-     case MDC_PRONE_HEADFIRST_SAGITTAL    :
-     case MDC_PRONE_HEADFIRST_CORONAL     : 
-      strcpy(mdcbufr,"head_in"); break;
-     case MDC_SUPINE_FEETFIRST_TRANSVERSAL:
-     case MDC_SUPINE_FEETFIRST_SAGITTAL   :
-     case MDC_SUPINE_FEETFIRST_CORONAL    :
-     case MDC_PRONE_FEETFIRST_TRANSVERSAL :
-     case MDC_PRONE_FEETFIRST_SAGITTAL    : 
-     case MDC_PRONE_FEETFIRST_CORONAL     :
-      strcpy(mdcbufr,"feet_in"); break;
-     default                              :
-      strcpy(mdcbufr,"unknown"); break;
-  }
-
-  return(mdcbufr);
-}
-
 char *MdcGetStrSliceOrient(int patient_slice_orient)
 {
   switch (patient_slice_orient) {
-     case MDC_SUPINE_HEADFIRST_TRANSVERSAL:
-     case MDC_PRONE_HEADFIRST_TRANSVERSAL :
-     case MDC_SUPINE_FEETFIRST_TRANSVERSAL:
-     case MDC_PRONE_FEETFIRST_TRANSVERSAL :
+     case MDC_SUPINE_HEADFIRST_TRANSAXIAL:
+     case MDC_PRONE_HEADFIRST_TRANSAXIAL :
+     case MDC_SUPINE_FEETFIRST_TRANSAXIAL:
+     case MDC_PRONE_FEETFIRST_TRANSAXIAL :
       strcpy(mdcbufr,"Transverse"); break;
      case MDC_SUPINE_HEADFIRST_SAGITTAL   :
      case MDC_PRONE_HEADFIRST_SAGITTAL    :
@@ -709,6 +921,89 @@ char *MdcGetStrMotion(int motion)
   return(mdcbufr);
 }
 
+char *MdcGetStrModality(int modint)
+{
+  switch (modint) {
+    case M_PT: return("PT");
+    case M_NM: return("NM");
+    default  : return("NM");
+  }
+}
+
+char *MdcGetStrGSpectNesting(int nesting)
+{
+  switch (nesting) {
+    case MDC_GSPECT_NESTING_SPECT: return("SPECT");
+    case MDC_GSPECT_NESTING_GATED: return("Gated");
+    default : return("unknown");
+  }
+}
+
+char *MdcGetStrHHMMSS(float msecs)
+{
+  unsigned int s, ms, hrs, mins, secs;
+
+  s = (unsigned int)(msecs / 1000.);
+
+  ms = (unsigned int)(msecs - (s * 1000.));
+
+  hrs  = s / 3600; s -= hrs  * 3600;
+
+  mins = s / 60;   s -= mins * 60.;
+
+  secs = s;
+
+  if (hrs > 0) {
+    sprintf(mdcbufr,"%02uh%02um%02u",hrs,mins,secs);
+  }else if (mins > 0) {
+    sprintf(mdcbufr,"%02um%02u",mins,secs);
+  }else{
+    sprintf(mdcbufr,"%02us%03u",secs,ms);
+  }
+
+  return(mdcbufr);
+}
+
+int MdcGetIntModality(char *modstr)
+{
+  int modint;
+
+  modint = (modstr[0]<<8)|modstr[1];
+
+  return(modint);
+}
+
+int MdcGetIntSliceOrient(int patient_slice_orient)
+{
+  int slice_orient;
+
+  switch (patient_slice_orient) {
+     case MDC_SUPINE_HEADFIRST_TRANSAXIAL :
+     case MDC_PRONE_HEADFIRST_TRANSAXIAL  :
+     case MDC_SUPINE_FEETFIRST_TRANSAXIAL :
+     case MDC_PRONE_FEETFIRST_TRANSAXIAL  :
+      slice_orient = MDC_TRANSAXIAL;
+      break;
+     case MDC_SUPINE_HEADFIRST_SAGITTAL   :
+     case MDC_PRONE_HEADFIRST_SAGITTAL    :
+     case MDC_SUPINE_FEETFIRST_SAGITTAL   :
+     case MDC_PRONE_FEETFIRST_SAGITTAL    :
+      slice_orient = MDC_SAGITTAL;
+      break;
+     case MDC_SUPINE_HEADFIRST_CORONAL    :
+     case MDC_PRONE_HEADFIRST_CORONAL     :
+     case MDC_SUPINE_FEETFIRST_CORONAL    :
+     case MDC_PRONE_FEETFIRST_CORONAL     :
+      slice_orient = MDC_CORONAL;
+      break;
+     default                              : 
+      slice_orient = MDC_TRANSAXIAL;
+  }
+
+  return(slice_orient);
+
+}
+
 const char *MdcGetLibLongVersion(void)
 {
   return(MDC_LIBVERS);
@@ -748,13 +1043,19 @@ int MdcMakeScanInfoStr(FILEINFO *fi)
 Short Patient/Scan Information\n\
 ******************************\n");
   size = (Uint32)strlen(mdcbufr);
-  sprintf(strbuf,"Patient Name: %s\n",fi->patient_name);
+  sprintf(strbuf,"Patient Name  : %s\n",fi->patient_name);
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
   else return MDC_NO;
-  sprintf(strbuf,"Patient Sex : %s\n",fi->patient_sex);
+  sprintf(strbuf,"Patient Sex   : %s\n",fi->patient_sex);
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
   else return MDC_NO;
-  sprintf(strbuf,"Patient ID  : %s\n",fi->patient_id);
+  sprintf(strbuf,"Patient ID    : %s\n",fi->patient_id);
+  if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
+  else return MDC_NO;
+  sprintf(strbuf,"Patient DOB   : %s\n",fi->patient_dob);
+  if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
+  else return MDC_NO;
+  sprintf(strbuf,"Patient Weight: %.2f\n",fi->patient_weight);
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
   else return MDC_NO;
   sprintf(strbuf,"Study Date  : %02d/%02d/%04d\n",fi->study_date_day
@@ -767,7 +1068,7 @@ Short Patient/Scan Information\n\
                                                ,fi->study_time_second);
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
   else return MDC_NO;
-  sprintf(strbuf,"Study Name  : %s\n",fi->study_name);
+  sprintf(strbuf,"Study ID    : %s\n",fi->study_id);
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
   else return MDC_NO;
   sprintf(strbuf,"Study Descr : %s\n",fi->study_descr);
@@ -797,6 +1098,9 @@ Short Patient/Scan Information\n\
                                        MdcGetStrYesNo(fi->flood_corrected));
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
   else return MDC_NO;
+  sprintf(strbuf,"Series Description   : %s\n",fi->series_descr);
+  if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
+  else return MDC_NO;
   sprintf(strbuf,"Radiopharmaceutical  : %s\n",fi->radiopharma);
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
   else return MDC_NO;
@@ -808,6 +1112,10 @@ Short Patient/Scan Information\n\
                                        fi->isotope_halflife);
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
   else return MDC_NO;
+  sprintf(strbuf,"Injected Dose        : %+e [MBq]\n",
+                                       fi->injected_dose);
+  if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
+  else return MDC_NO;
   sprintf(strbuf,"Gantry Tilt          : %+e degrees\n",
                                        fi->gantry_tilt);
   if ((size=MdcCheckStrSize(strbuf,size,0))) strcat(mdcbufr,strbuf);
@@ -827,13 +1135,13 @@ int MdcIsDigit(char c)
 void MdcWaitForEnter(int page)
 {
   if (page > 0) {
-    printf("\t\t*********** Press <enter> for page #%d **********",page);
+    MdcPrntScrn("\t\t*********** Press <enter> for page #%d **********",page);
   }
   if (page == 0) {
-    printf("\t\t********** Press <enter> for next page **********");
+    MdcPrntScrn("\t\t********** Press <enter> for next page **********");
   }
   if (page < 0 ) {
-    printf("Press <enter> to continue ...");
+    MdcPrntScrn("Press <enter> to continue ...");
   }
   while ( fgetc(stdin) != '\n' ) { /* wait until <enter> key pressed */ }
 }
@@ -842,11 +1150,11 @@ Int32 MdcGetSelectionType(void)
 {
    Int32 type=-1;
 
-   printf("\n\tSelection Type:\n");
-   printf("\n\ttype  %d  ->  normal",MDC_INPUT_NORM_STYLE);
-   printf("\n\t      %d  ->  ecat\n",MDC_INPUT_ECAT_STYLE);
-   printf("\n\tYour choice [%d]? ",MDC_INPUT_NORM_STYLE);
-   fgets(mdcbufr,MDC_2KB_OFFSET-1,stdin);
+   MdcPrntScrn("\n\tSelection Type:\n");
+   MdcPrntScrn("\n\ttype  %d  ->  normal",MDC_INPUT_NORM_STYLE);
+   MdcPrntScrn("\n\t      %d  ->  ecat\n",MDC_INPUT_ECAT_STYLE);
+   MdcPrntScrn("\n\tYour choice [%d]? ",MDC_INPUT_NORM_STYLE);
+   MdcGetStrLine(mdcbufr,MDC_2KB_OFFSET-1,stdin);
 
    type=(Int32)atol(mdcbufr);
 
