@@ -7,19 +7,20 @@
  *                                                                         *
  * project      : (X)MedCon by Erik Nolf                                   *
  *                                                                         *
- * Functions    : MdcReadINW()          - Read INW file                    *
+ * Functions    : MdcCheckINW()         - Check INW format                 *
+ *                MdcReadINW()          - Read INW file                    *
  *                MdcWriteHeadStart()   - Write Head_start                 *
  *                MdcWriteHeadGen()     - Write Head_gen                   *
- *                MdcSkipHeadSpec()     - Skip Head_spec in file           *
+ *                MdcSkipHeadSpecs()    - Skip  Head_specs in file         *
  *                MdcWriteHeadSpecs()   - Write Head_specs                 *
  *                MdcWriteINW()         - Write INW file                   *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-inw.c,v 1.1.1.1 2000/10/28 16:51:21 enlf Exp $
+/* $Id: m-inw.c,v 1.33 2004/05/09 22:48:03 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -39,6 +40,21 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
+
 #include "medcon.h"
 
 /****************************************************************************
@@ -56,7 +72,7 @@ int MdcCheckINW(FILEINFO *fi)
 
   MdcSWAP(hs.mark);
 
-  if (hs.mark != MDC_INW_SIG) return(MDC_FRMT_BAD);
+  if (hs.mark != MDC_INW_SIG) return(MDC_FRMT_NONE);
 
   return(MDC_FRMT_INW);
 }
@@ -69,15 +85,17 @@ char *MdcReadINW(FILEINFO *fi)
   MDC_INW_HEAD_GEN   hg;
   MDC_INW_HEAD_SPEC  *hsp;
   IMG_DATA   *id;
-  Uint32 i, bytes; 
+  Uint32 i, bytes, number; 
   char *err=NULL;
 
-#if XSUPPORTED
-  if (XMDC_MEDCON) XMdcBeginProgressBar("Reading INW:");
-#endif
+  if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_BEGIN,0.,"Reading INW:");
 
   if (MDC_VERBOSE) MdcPrntMesg("INW  Reading <%s> ...",fi->ifname);
 
+  if (MDC_ECHO_ALIAS == MDC_YES) {
+    MdcEchoAliasName(fi); return(NULL); /* useless here */
+  }
+
   memset(&hs,0,MDC_INW_HEAD_START_SIZE);
   memset(&hg,0,MDC_INW_HEAD_GEN_SIZE);
 
@@ -106,50 +124,54 @@ char *MdcReadINW(FILEINFO *fi)
 
   if (MDC_INFO) {
 
-    printf("\nHEAD START (%d bytes)\n",MDC_INW_HEAD_START_SIZE);
+    MdcPrntScrn("\nHEAD START (%d bytes)\n",MDC_INW_HEAD_START_SIZE);
     MdcPrintLine('-',MDC_HALF_LENGTH);
-    printf("mark          : 0x%x\n",hs.mark);
-    printf("version       : %hd.%hd\n",hs.version/256
+    MdcPrntScrn("mark          : 0x%x\n",hs.mark);
+    MdcPrntScrn("version       : %hd.%hd\n",hs.version/256
                                       ,hs.version%256);
-    printf("size_header   : %hd\n",hs.size_header);
-    printf("size_start    : %hd\n",hs.size_start);
-    printf("size_gen      : %hd\n",hs.size_gen);
-    printf("size_spec     : %hd\n",hs.size_spec);
-    printf("reserved      : %.10s",hs.reserved);
-    printf("\n");
+    MdcPrntScrn("size_header   : %hd\n",hs.size_header);
+    MdcPrntScrn("size_start    : %hd\n",hs.size_start);
+    MdcPrntScrn("size_gen      : %hd\n",hs.size_gen);
+    MdcPrntScrn("size_spec     : %hd\n",hs.size_spec);
+    MdcPrntScrn("reserved      : %.10s",hs.reserved);
+    MdcPrntScrn("\n");
 
-    printf("\nHEAD GEN   (%d bytes)\n",MDC_INW_HEAD_GEN_SIZE);
+    MdcPrntScrn("\nHEAD GEN   (%d bytes)\n",MDC_INW_HEAD_GEN_SIZE);
     MdcPrintLine('-',MDC_HALF_LENGTH); 
-    printf("number planes : %hd\n",hg.no);
-    printf("number columns: %hd\n",hg.sizeX);
-    printf("number rows   : %hd\n",hg.sizeY);
-    printf("pixel type    : %hd\n",hg.pixel_type);
-    printf("init transl   : %hd [mm]\n",hg.init_trans);
-    printf("dummy1        : %hd\n",hg.dummy1);
-    printf("day           : %.12s\n",hg.day);
-    printf("time          : %d [sec]\n",hg.time);
-    printf("decay constant: %+e\n",hg.decay_cst);
-    printf("pixel size    : %+e [mm]\n",hg.pixel_size);
-    printf("scaled max    : %+e\n",hg.max);
-    printf("scaled min    : %+e\n",hg.min);
-    printf("scanner type  : %hd ",hg.scanner);
+    MdcPrntScrn("number planes : %hd\n",hg.no);
+    MdcPrntScrn("number columns: %hd\n",hg.sizeX);
+    MdcPrntScrn("number rows   : %hd\n",hg.sizeY);
+    MdcPrntScrn("pixel type    : %hd\n",hg.pixel_type);
+    MdcPrntScrn("init transl   : %hd [mm]\n",hg.init_trans);
+    MdcPrntScrn("dummy1        : %hd\n",hg.dummy1);
+    MdcPrntScrn("day           : %.12s\n",hg.day);
+    MdcPrntScrn("time          : %d [sec]\n",hg.time);
+    MdcPrntScrn("decay constant: %+e\n",hg.decay_cst);
+    MdcPrntScrn("pixel size    : %+e [mm]\n",hg.pixel_size);
+    MdcPrntScrn("scaled max    : %+e\n",hg.max);
+    MdcPrntScrn("scaled min    : %+e\n",hg.min);
+    MdcPrntScrn("scanner type  : %hd ",hg.scanner);
     switch (hg.scanner) {
-      case  EcatII: printf("(EcatII)");        break;
-      case  EcatIV: printf("(EcatIV)");        break;
-      default     : printf("(Unknown)"); 
+      case  EcatII: MdcPrntScrn("(EcatII)");        break;
+      case  EcatIV: MdcPrntScrn("(EcatIV)");        break;
+      default     : MdcPrntScrn("(Unknown)"); 
     }
-    printf("\n");
-    printf("recon type    : %c ",hg.reconstruction);
+    MdcPrntScrn("\n");
+    MdcPrntScrn("recon type    : %c ",hg.reconstruction);
     switch (hg.reconstruction) {
-      case reconFBP     : printf("(Filtered Backprojection)"); break;
-      case reconMaxLikFV: printf("(Maximum likelihood F. Vermeulen)"); break;
-      case reconMaxLik  : printf("(Maximum likelihood T. De Backer)"); break;
-      case reconMaxPos  : printf("(Maximum a Posteriori)"); break;
-      default           : printf("(Unknown)");
+      case reconFBP     : MdcPrntScrn("(Filtered Backprojection)");
+          break;
+      case reconMaxLikFV: MdcPrntScrn("(Maximum likelihood F. Vermeulen)");
+          break;
+      case reconMaxLik  : MdcPrntScrn("(Maximum likelihood T. De Backer)");
+          break;
+      case reconMaxPos  : MdcPrntScrn("(Maximum a Posteriori)");
+          break;
+      default           : MdcPrntScrn("(Unknown)");
     }
-    printf("\n");
-    printf("recon version : %d\n",(int)hg.recon_version);
-    printf("reserved      : %.24s\n",hg.reserved);
+    MdcPrntScrn("\n");
+    MdcPrntScrn("recon version : %d\n",(int)hg.recon_version);
+    MdcPrntScrn("reserved      : %.24s\n",hg.reserved);
   }
 
   /* check some supported things */
@@ -160,8 +182,8 @@ char *MdcReadINW(FILEINFO *fi)
     return("INW  Unsupported version"); 
 
   /* fill in the FILEINFO struct */
-  fi->number = hg.no;
-  if (fi->number == 0 ) return("INW  No valid images specified");
+  number = hg.no;
+  if (number == 0 ) return("INW  No valid images specified");
   fi->mwidth = hg.sizeX;
   fi->mheight = hg.sizeY;
   fi->bits   = hg.pixel_type * 8;
@@ -200,28 +222,24 @@ char *MdcReadINW(FILEINFO *fi)
 
   /* now we read the spec/image headers */
 
-  if ((hsp = (MDC_INW_HEAD_SPEC *)malloc(MDC_INW_HEAD_SPEC_SIZE*fi->number)) 
+  if ((hsp = (MDC_INW_HEAD_SPEC *)malloc(MDC_INW_HEAD_SPEC_SIZE*number)) 
            == NULL)
     return("INW  Bad malloc HeadSpec structs");
 
-  memset(hsp,0,MDC_INW_HEAD_SPEC_SIZE*fi->number);
+  memset(hsp,0,MDC_INW_HEAD_SPEC_SIZE*number);
 
-  if (fread((char *)hsp,MDC_INW_HEAD_SPEC_SIZE,fi->number,fi->ifp) 
-    != fi->number) {
+  if (fread((char *)hsp,MDC_INW_HEAD_SPEC_SIZE,number,fi->ifp) != number) {
     MdcFree(hsp);
     return("INW  Bad read HeadSpec structs");
   }
 
-  if (!MdcGetStructID(fi)) return("INW  Bad malloc IMG_DATA structs");
+  if (!MdcGetStructID(fi,number))
+    return("INW  Bad malloc IMG_DATA structs");
 
   for (i=0; i<fi->number; i++) {
 
-#if XSUPPORTED
-     if (XMDC_MEDCON) {
-       pvalue += 1./(float)fi->number;
-       XMdcUpdateProgressBar(NULL);
-     }
-#endif
+     if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_INCR,1./(float)fi->number,NULL);
+
      id = &fi->image[i];
 
      MdcSWAP(hsp[i].time);     MdcSWAP(hsp[i].trans);
@@ -230,15 +248,16 @@ char *MdcReadINW(FILEINFO *fi)
      MdcMakeIEEEfl(hsp[i].cal_cst);
 
      if (MDC_INFO) {
-       printf("\n");
-       printf("\nHEAD SPEC #%.3d (%d bytes)\n",i+1,MDC_INW_HEAD_SPEC_SIZE);
+       MdcPrntScrn("\n");
+       MdcPrntScrn("\nHEAD SPEC #%.3d (%d bytes)\n",i+1
+                                                      ,MDC_INW_HEAD_SPEC_SIZE);
        MdcPrintLine('-',MDC_HALF_LENGTH);
-       printf("relative time : %d\n",hsp[i].time);
-       printf("calibr const  : %+e\n",hsp[i].cal_cst);
-       printf("pixel max     : %d\n",hsp[i].max);
-       printf("pixel min     : %d\n",hsp[i].min);
-       printf("relat transl  : %hd [mm]\n",hsp[i].trans);
-       printf("reserved      : %.6s\n",hsp[i].reserved);
+       MdcPrntScrn("relative time : %d\n",hsp[i].time);
+       MdcPrntScrn("calibr const  : %+e\n",hsp[i].cal_cst);
+       MdcPrntScrn("pixel max     : %d\n",hsp[i].max);
+       MdcPrntScrn("pixel min     : %d\n",hsp[i].min);
+       MdcPrntScrn("relat transl  : %hd [mm]\n",hsp[i].trans);
+       MdcPrntScrn("reserved      : %.6s\n",hsp[i].reserved);
      }
 
      if (hsp[i].trans < 0.0 ) hsp[i].trans = - hsp[i].trans; /* positive */
@@ -286,20 +305,22 @@ char *MdcReadINW(FILEINFO *fi)
     fi->pixdim[3] /= (float)fi->number; /* average */
   }
 
+  /* fill in orientation information */
+  fi->pat_slice_orient = MDC_SUPINE_HEADFIRST_TRANSAXIAL; /* default! */
+  strcpy(fi->pat_pos,MdcGetStrPatPos(fi->pat_slice_orient));
+  strcpy(fi->pat_orient,MdcGetStrPatOrient(fi->pat_slice_orient));
+
   /* fill in the Acr/Nema variables */
   for (i=0; i<fi->number; i++) {
      id = &fi->image[i];
-     id->pat_slice_orient = MDC_SUPINE_HEADFIRST_TRANSVERSAL; /* default! */
      id->slice_spacing = id->slice_width;
-     strcpy(id->pat_pos,MdcGetStrPatientPos(id->pat_slice_orient));
-     strcpy(id->pat_orient,MdcGetStrPatientOrient(id->pat_slice_orient));
      MdcFillImgPos(fi,i,i,0.0);
      MdcFillImgOrient(fi,i);
   } 
 
   MdcFree(hsp);
 
-  MdcFileClose(fi->ifp);
+  MdcCloseFile(fi->ifp);
 
   if (fi->truncated) return("INW  Truncated image file");
 
@@ -407,7 +428,7 @@ int MdcWriteHeadSpecs(FILEINFO *fi)
      }else{
        hsp.max     = (Int16)id->max;
        hsp.min     = (Int16)id->min;
-       hsp.cal_cst = id->rescale_fctr;
+       hsp.cal_cst = id->rescale_slope;
      }
 
      hsp.trans = img * (Int16)id->slice_width;
@@ -415,7 +436,7 @@ int MdcWriteHeadSpecs(FILEINFO *fi)
      if (fi->number > 1) hsp.trans = img * (Int16)id->slice_spacing;
 #endif
 
-     memcpy(hsp.reserved,MDC_SHORTINSTIT,6);
+     memcpy(hsp.reserved,MDC_INSTITUTION,6);
 
      MdcMakeVAXfl(hsp.cal_cst);
 
@@ -444,30 +465,30 @@ char *MdcWriteINW(FILEINFO *fi)
 
   if (MDC_FORCE_INT != MDC_NO) {
     if (MDC_FORCE_INT != BIT16_S) {
-#if XSUPPORTED
-      if (XMDC_MEDCON) {
-        XMdcDisplayWarn("INW  Only Int16 pixels supported");
-      }else
-#endif
-      {
       MdcPrntWarn("INW  Only Int16 pixels supported");
-      }
     }
   } 
 
-  if (XMDC_MEDCON == MDC_NO)
-    MdcDefaultName(MDC_FRMT_INW,fi->ofname,fi->ifname);
+  if (XMDC_GUI == MDC_NO) {
+    MdcDefaultName(fi,MDC_FRMT_INW,fi->ofname,fi->ifname);
+  }
 
-#if XSUPPORTED
-  if (XMDC_MEDCON) XMdcBeginProgressBar("Writing INW:");
-#endif
+  if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_BEGIN,0.,"Writing INW:");
 
   if (MDC_VERBOSE) MdcPrntMesg("INW  Writing <%s> ...",fi->ofname);
 
-  if (MdcFileExists(fi->ofname))
-    return("INW  File exists!!");
-  if ( (fi->ofp=fopen(fi->ofname,"wb")) == NULL)
-    return("INW  Couldn't open file");
+  /* check for colored files */
+  if (fi->map == MDC_MAP_PRESENT)
+    return("INW  Colored files unsupported");
+
+  if (MDC_FILE_STDOUT == MDC_YES) {
+    fi->ofp = stdout;
+  }else{
+    if (MdcKeepFile(fi->ofname))
+      return("INW  File exists!!");
+    if ( (fi->ofp=fopen(fi->ofname,"wb")) == NULL)
+      return("INW  Couldn't open file");
+  }
 
   if ( !MdcWriteHeadStart(fi) )return("INW  Bad write HeadStart struct");
  
@@ -477,18 +498,17 @@ char *MdcWriteINW(FILEINFO *fi)
 
   /* write the images */
   for (i=0; i<fi->number; i++) {
-#if XSUPPORTED
-     if (XMDC_MEDCON) {
-       pvalue += 1./(float)fi->number;
-       XMdcUpdateProgressBar(NULL);
-     }
-#endif
+
+     if (MDC_PROGRESS) MdcProgress(MDC_PROGRESS_INCR,1./(float)fi->number,NULL);
 
      id = &fi->image[i];
 
-     switch (id->type) { 
-       case BIT16_S: buf = id->buf; FREE=MDC_NO; break;
-       default     : buf = MdcGetImgBIT16_S(fi, i); FREE=MDC_YES; 
+     if ((id->type != BIT16_S) || MDC_QUANTIFY || MDC_CALIBRATE) {
+       buf = MdcGetImgBIT16_S(fi, i);
+       FREE=MDC_YES; type=BIT16_S;
+     }else{
+       buf = id->buf; 
+       FREE=MDC_NO; type=id->type;
      }
  
      if (buf == NULL) return("INW  Bad malloc image buffer");
@@ -523,7 +543,9 @@ char *MdcWriteINW(FILEINFO *fi)
 
   if ( !MdcWriteHeadSpecs(fi) ) return("INW  Bad write HeadSpecs structs");
 
-  MdcFileClose(fi->ofp);
+  MdcCheckQuantitation(fi);
+
+  MdcCloseFile(fi->ofp);
 
   return NULL; 
 }
