@@ -14,11 +14,11 @@
  *                MdcPrintPixel()       - Print pixel value                *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: m-pixels.c,v 1.1.1.1 2000/10/28 16:51:09 enlf Exp $
+/* $Id: m-pixels.c,v 1.29 2004/01/04 22:36:16 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -38,6 +38,21 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
+
 #include "medcon.h"
 
 /****************************************************************************
@@ -50,17 +65,21 @@ void MdcDisplayPixels(FILEINFO *fi)
    Uint32 *cols=NULL;
    Uint32 *rows=NULL;
 
-   
+   if (MDC_FILE_STDIN == MDC_YES) return; /* stdin already in use */
+
+   if (fi->type == COLRGB) {
+     MdcPrntWarn("Print values of true color files unsupported");
+     return;
+   }
 
    MdcPrintLine('-',MDC_FULL_LENGTH);
-   printf("\tPIXEL DISPLAY\t\tFILE: %s\n",fi->ifname);
+   MdcPrntScrn("\tPIXEL DISPLAY\t\tFILE: %s\n",fi->ifname);
    MdcPrintLine('-',MDC_FULL_LENGTH);
 
    if (MdcAskPixels(fi,&images,&cols,&rows) == MDC_YES) {
-     MdcPrintLine('+',MDC_FULL_LENGTH);
      MdcGetPixels(fi,images,cols,rows);
-     MdcPrintLine('+',MDC_FULL_LENGTH);
    }
+
    MdcPrintLine('-',MDC_FULL_LENGTH);
 
    MdcFree(images);
@@ -75,13 +94,39 @@ int MdcAskPixels(FILEINFO *fi, Uint32 *img[], Uint32 *col[], Uint32 *row[])
    Uint32 images=1, bt, it, f, p, g, b;
    Uint32  *frames, *planes, *gates, *beds;
    Uint32 *itmp, *ctmp, *rtmp;
+   char *msg=NULL;
+
+   if (MDC_PIXELS_PRINT_ALL == MDC_YES) {
+     itmp = (Uint32 *)malloc(2*sizeof(Uint32));
+     ctmp = (Uint32 *)malloc(2*sizeof(Uint32));
+     rtmp = (Uint32 *)malloc(2*sizeof(Uint32));
+
+     if ((itmp == NULL) || (ctmp == NULL) || (rtmp == NULL)) {
+       MdcPrntWarn("Failure to malloc index buffers");
+       return(MDC_NO);
+     }
+
+     /* all images */
+     itmp[0]=1;  itmp[1]=0;
+     /* one pixel coord: 0,0 = all */
+     ctmp[0]=1; ctmp[1]=0;
+     rtmp[0]=1; rtmp[1]=0;
+
+     *img = itmp;
+     *col = ctmp;
+     *row = rtmp;
+
+     return(MDC_YES);
+   }
+
 
    a1 = MdcGetSelectionType();
 
-   printf("\n");
-   printf("\n\tInput notes: a) Finish the list with a negative number");
-   printf("\n\t             b) Any number must be one-based (0 = All)");
-   printf("\n\t             c) The syntax of a range: X...Y\n");
+   MdcPrntScrn("\n");
+   MdcPrntScrn("\n\tInput notes: a) Any number must be one-based (0 = All)");
+   MdcPrntScrn("\n\t             b) Syntax of range   : X...Y or X-Y");
+   MdcPrntScrn("\n\t             c) Syntax of interval: X:S:Y    (S = step)");
+   MdcPrntScrn("\n\t             d) Just type <enter> for the entire range\n");
 
    if ( a1 == MDC_INPUT_ECAT_STYLE ) { /* ecat   */
      if ( (planes=(Uint32 *)malloc((fi->dim[3]+1)*sizeof(Uint32))) == NULL ){
@@ -111,39 +156,39 @@ int MdcAskPixels(FILEINFO *fi, Uint32 *img[], Uint32 *col[], Uint32 *row[])
      }
      memset(beds,0,(fi->dim[6]+1)*sizeof(Uint32));
 
-     printf("\n\tGive planes list (1...%u): ",fi->dim[3]);
+     MdcPrntScrn("\n\tGive planes list [1...%u]: ",fi->dim[3]);
      MdcGetStrInput(mdcbufr,MDC_2KB_OFFSET);
  
-     if (MdcHandleEcatList(mdcbufr,&planes,fi->dim[3]) != MDC_OK) {
-       MdcFree(frames); MdcFree(planes);
-       MdcFree(gates); MdcFree(beds);
+     if ((msg=MdcHandleEcatList(mdcbufr,&planes,(Uint32)fi->dim[3])) != NULL) {
+       MdcPrntWarn(msg);
+       MdcFree(frames); MdcFree(planes); MdcFree(gates); MdcFree(beds);
        return(MDC_NO);
      }
 
-     printf("\n\tGive frames list (1...%u): ",fi->dim[4]);
+     MdcPrntScrn("\n\tGive frames list [1...%u]: ",fi->dim[4]);
      MdcGetStrInput(mdcbufr,MDC_2KB_OFFSET);
      
-     if (MdcHandleEcatList(mdcbufr,&frames,fi->dim[4]) != MDC_OK) {
-       MdcFree(frames); MdcFree(planes);
-       MdcFree(gates); MdcFree(beds);
+     if ((msg=MdcHandleEcatList(mdcbufr,&frames,(Uint32)fi->dim[4])) != NULL) {
+       MdcPrntWarn(msg);
+       MdcFree(frames); MdcFree(planes); MdcFree(gates); MdcFree(beds);
        return(MDC_NO);
      }
 
-     printf("\n\tGive gates  list (1...%u): ",fi->dim[5]);
+     MdcPrntScrn("\n\tGive gates  list [1...%u]: ",fi->dim[5]);
      MdcGetStrInput(mdcbufr,MDC_2KB_OFFSET);
 
-     if (MdcHandleEcatList(mdcbufr,&gates,fi->dim[5]) != MDC_OK) {
-       MdcFree(frames); MdcFree(planes);
-       MdcFree(gates); MdcFree(beds);
+     if ((msg=MdcHandleEcatList(mdcbufr,&gates,(Uint32)fi->dim[5])) != NULL) {
+       MdcPrntWarn(msg);
+       MdcFree(frames); MdcFree(planes); MdcFree(gates); MdcFree(beds);
        return(MDC_NO);
      }
 
-     printf("\n\tGive beds   list (1...%u): ",fi->dim[6]);
+     MdcPrntScrn("\n\tGive beds   list [1...%u]: ",fi->dim[6]);
      MdcGetStrInput(mdcbufr,MDC_2KB_OFFSET);
 
-     if (MdcHandleEcatList(mdcbufr,&beds,fi->dim[6]) != MDC_OK) {
-       MdcFree(frames); MdcFree(planes);
-       MdcFree(gates); MdcFree(beds);
+     if ((msg=MdcHandleEcatList(mdcbufr,&beds,(Uint32)fi->dim[6])) != NULL) {
+       MdcPrntWarn(msg);
+       MdcFree(frames); MdcFree(planes); MdcFree(gates); MdcFree(beds);
        return(MDC_NO);
      }
 
@@ -179,7 +224,7 @@ int MdcAskPixels(FILEINFO *fi, Uint32 *img[], Uint32 *col[], Uint32 *row[])
      }
  
      if ((it-1) != images) {
-       MdcPrntErr(MDC_BAD_CODE,"Internal Error");
+       MdcPrntErr(MDC_BAD_CODE,"Internal Error ## Improper list handling");
      }
 
      MdcFree(planes); MdcFree(frames); MdcFree(gates); MdcFree(beds);
@@ -193,13 +238,14 @@ int MdcAskPixels(FILEINFO *fi, Uint32 *img[], Uint32 *col[], Uint32 *row[])
 
      itmp[0] = 0;
 
-     printf("\n\tGive a list of image numbers:     ex. 1 7...31 84 -1");
-     printf("\n\tYour input [1...%u]: ",fi->number);
+     MdcPrntScrn("\n\tGive a list of image numbers:     ex. 1 7...31 84");
+     MdcPrntScrn("\n\tYour input [1...%u]: ",fi->number);
      MdcGetStrInput(mdcbufr,MDC_2KB_OFFSET);
 
      it = 1; bt = 2;
 
-     if (MdcHandleNormList(mdcbufr,&itmp,&it,&bt,fi->number) != MDC_OK){
+     if ((msg=MdcHandleNormList(mdcbufr,&itmp,&it,&bt,fi->number)) != NULL){
+       MdcPrntWarn(msg);
        if (itmp != NULL) MdcFree(itmp);
        return(MDC_NO);
      }
@@ -226,23 +272,21 @@ int MdcAskPixels(FILEINFO *fi, Uint32 *img[], Uint32 *col[], Uint32 *row[])
    }
    if ( (rtmp=(Uint32 *)malloc(MDC_BUF_ITMS*sizeof(Uint32))) == NULL ) {
      MdcPrntWarn("Couldn't allocate pixels row buffer");
-     MdcFree(itmp);
-     MdcFree(ctmp);
+     MdcFree(itmp); MdcFree(ctmp);
      return(MDC_NO);
    }
  
    it=1; bt=2;
 
-   printf("\n\n\tGive a list of pixels x,y :       ex. 1,1 12,0 -1");
-   printf("\n\tYour input [%u,%u]: ",fi->mwidth,fi->mheight);
+   MdcPrntScrn("\n\n\tGive a list of pixels x,y :       ex. 1,1 12,0");
+   MdcPrntScrn("\n\tYour input [%u,%u]: ",fi->mwidth,fi->mheight);
    MdcGetStrInput(mdcbufr,MDC_2KB_OFFSET);
 
-   printf("\n");
+   MdcPrntScrn("\n");
 
-   if (MdcHandlePixelList(mdcbufr,&ctmp,&rtmp,&it,&bt) != MDC_OK) {
-     MdcFree(itmp);
-     MdcFree(ctmp);
-     MdcFree(rtmp);
+   if ((msg=MdcHandlePixelList(mdcbufr,&ctmp,&rtmp,&it,&bt)) != NULL) {
+     MdcPrntWarn(msg);
+     MdcFree(itmp); MdcFree(ctmp); MdcFree(rtmp);
      return(MDC_NO);
    }
  
@@ -250,9 +294,7 @@ int MdcAskPixels(FILEINFO *fi, Uint32 *img[], Uint32 *col[], Uint32 *row[])
 
    if ((ctmp[0] == 0) || (rtmp[0] == 0)) {
      MdcPrntWarn("No valid pixel specified!"); 
-     MdcFree(itmp);
-     MdcFree(ctmp);
-     MdcFree(rtmp);
+     MdcFree(itmp); MdcFree(ctmp); MdcFree(rtmp);
      return(MDC_NO);
    }
 
@@ -266,18 +308,29 @@ int MdcAskPixels(FILEINFO *fi, Uint32 *img[], Uint32 *col[], Uint32 *row[])
 void MdcGetPixels(FILEINFO *fi, Uint32 img[], Uint32 col[], Uint32 row[])
 {
 
-   Uint32 it, pt, ct, rt, number;
+   Uint32 it, pt, ct, rt, number, itotal;
    IMG_DATA *id;
 
-   for (it=1; it <= img[0]; it++) {
+   MdcPrintLine('+',MDC_FULL_LENGTH);
+   MdcPrntScrn("\
+ : image: :     slope     : :   intercept   :   pixel  :     value\n");
+   MdcPrintLine('+',MDC_FULL_LENGTH);
+
+   if (img[1] == 0) { /* special: just all images selected */
+     itotal = fi->number;
+   }else{             /* normal : get selected images from first index */
+     itotal = img[0];
+   }
+
+   for (it=1; it <= itotal; it++) {
       
-      if (img[1] == 0)  { /* all images selected, special case */
+      if (img[1] == 0) {  /* special: calc next image number */
         number = it-1;
-        id = &fi->image[number];
-      }else{
+      }else{              /* normal : get image number from next index */
         number = img[it] - 1;
-        id = &fi->image[number];
       }
+ 
+      id = &fi->image[number];
 
       for (pt=1; pt <= row[0]; pt++) {
 
@@ -313,6 +366,8 @@ void MdcGetPixels(FILEINFO *fi, Uint32 img[], Uint32 col[], Uint32 row[])
 
    }
 
+   MdcPrintLine('+',MDC_FULL_LENGTH);
+
 }
 
 double MdcGetOnePixel(IMG_DATA *id, Uint32 i, Uint32 x, Uint32 y)
@@ -406,9 +461,10 @@ void MdcPrintPixel(IMG_DATA *id, Uint32 i, Uint32 x, Uint32 y)
 
      ppv = MdcGetOnePixel(id,i,x,y);  /* Plain Pixel Value */
  
-     printf("image: %4u :",i+1);
-     printf("rescale_fctr: %+e :",id->rescale_fctr);
-     printf("pixel(%3u,%3u): %+e\n",x+1,y+1,ppv);
+     MdcPrntScrn("#: %4u :",i+1);
+     MdcPrntScrn("S: %+e :",id->rescale_slope);
+     MdcPrntScrn("I: %+e :",id->rescale_intercept);
+     MdcPrntScrn("P(%3u,%3u): %+e\n",x+1,y+1,ppv);
 
    }else{
 
