@@ -20,11 +20,11 @@
  *                XMdcColGbcCorrectSel();         - GBC selection          *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xcolgbc.c,v 1.1.1.1 2000/10/28 16:51:40 enlf Exp $
+/* $Id: xcolgbc.c,v 1.12 2004/01/04 22:36:17 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -45,6 +45,21 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
+
 #include "xmedcon.h"
 
 /****************************************************************************
@@ -73,19 +88,20 @@ void XMdcColGbcCorrectUpdate(void)
   GtkWidget *area;
   GdkPixbuf *imtmp, *imnew;
   GdkGC *gc;
-  Uint32 w, h;
+  int w, h;
 
   area = sGbc.area;
   gc = area->style->white_gc;
 
   XMdcSetGbcCorrection(&modtmp);
 
-  imtmp = XMdcBuildGdkPixbuf(sGbc.img8,sGbc.w,sGbc.h,modtmp.vgbc,MDC_NO);
-  gdk_pixbuf_unref(sGbc.im);
+  imtmp = XMdcBuildGdkPixbuf(sGbc.img8,sGbc.w,sGbc.h,sGbc.t,modtmp.vgbc,MDC_NO);
 
   w = gdk_pixbuf_get_width(sGbc.im);
   h = gdk_pixbuf_get_height(sGbc.im);
 
+  gdk_pixbuf_unref(sGbc.im);
+
   imnew = gdk_pixbuf_scale_simple(imtmp,w,h,sRenderSelection.Interp);
   gdk_pixbuf_unref(imtmp); sGbc.im = imnew;
 
@@ -99,21 +115,21 @@ void XMdcColGbcCorrectMakeIcons(void)
 
   GdkPixbuf *im0, *im;
   
-  im0=gdk_pixbuf_new_from_data(xmdc_brightness_icon, GDK_COLORSPACE_RGB, FALSE,                                8, 12, 12, 3*12, NULL, NULL);
+  im0=gdk_pixbuf_new_from_data(xmdc_brightness_icon, GDK_COLORSPACE_RGB, FALSE,                                8, 12, 12, 3*12, (void(*)())NULL, NULL);
   im = gdk_pixbuf_add_alpha(im0,TRUE,0xff,0x00,0xff);
   gdk_pixbuf_render_pixmap_and_mask(im,&sGbc.brightness_pmap
                                       ,&sGbc.brightness_mask,254);
   gdk_pixbuf_unref(im); gdk_pixbuf_unref(im0);
 
   im0=gdk_pixbuf_new_from_data(xmdc_contrast_icon,   GDK_COLORSPACE_RGB, FALSE,
-                                8, 12, 12, 3*12, NULL, NULL);
+                               8, 12, 12, 3*12, (void(*)())NULL, NULL);
   im = gdk_pixbuf_add_alpha(im0,TRUE,0xff,0x00,0xff);
   gdk_pixbuf_render_pixmap_and_mask(im,&sGbc.contrast_pmap
                                       ,&sGbc.contrast_mask,254);
   gdk_pixbuf_unref(im); gdk_pixbuf_unref(im0);
 
   im0=gdk_pixbuf_new_from_data(xmdc_gamma_icon,      GDK_COLORSPACE_RGB, FALSE,
-                                8, 12, 12, 3*12, NULL, NULL);
+                               8, 12, 12, 3*12, (void(*)())NULL, NULL);
   im = gdk_pixbuf_add_alpha(im0,TRUE,0xff,0x00,0xff);
   gdk_pixbuf_render_pixmap_and_mask(im,&sGbc.gamma_pmap
                                       ,&sGbc.gamma_mask,254);
@@ -124,10 +140,10 @@ void XMdcColGbcCorrectMakeIcons(void)
 void XMdcColGbcCorrectAddImg(GtkWidget *w)
 {
   GtkWidget *box, *area, *imgbox;
-  Uint32 rw, rh;
+  int rw, rh;
 
-  rw = XMdcResize(sGbc.w);
-  rh = XMdcResize(sGbc.h);
+  rw = (signed)XMdcResize(sGbc.w);
+  rh = (signed)XMdcResize(sGbc.h);
 
   box = gtk_vbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(w), box, FALSE, FALSE, 0);
@@ -220,9 +236,9 @@ void XMdcColGbcCorrectAddSliders(GtkWidget *w)
   box = gtk_vbox_new(TRUE, 0);
   gtk_container_add(GTK_CONTAINER(frame), box);
   gtk_widget_show(box);
-  XMdcColGbcCorrectAddOneSlider(box, &modtmp.gamma, i1, &svgamma);
-  XMdcColGbcCorrectAddOneSlider(box, &modtmp.brightness, i2, &svbrightness);
-  XMdcColGbcCorrectAddOneSlider(box, &modtmp.contrast, i3, &svcontrast);
+  XMdcColGbcCorrectAddOneSlider(box,(int *)&modtmp.gamma,i1,&svgamma);
+  XMdcColGbcCorrectAddOneSlider(box,(int *)&modtmp.brightness,i2,&svbrightness);
+  XMdcColGbcCorrectAddOneSlider(box,(int *)&modtmp.contrast,i3,&svcontrast);
 
   XMdcColGbcCorrectAddImg(box0);
 
@@ -262,26 +278,29 @@ void XMdcColGbcCorrectSel(GtkWidget *widget, Uint32 nr)
   GtkWidget *button;
   GtkWidget *separator;
   GdkPixbuf *im;
-  Uint32 rw, rh;
+  int rw, rh;
 
   sGbc.nr = nr;
   sGbc.i = my.realnumber[nr];
   sGbc.w = my.fi->image[sGbc.i].width;
   sGbc.h = my.fi->image[sGbc.i].height;
+  sGbc.t = my.fi->image[sGbc.i].type;
   modtmp.gamma = sGbc.mod.gamma;
   modtmp.brightness = sGbc.mod.brightness;
   modtmp.contrast = sGbc.mod.contrast;
   XMdcSetGbcCorrection(&modtmp);
-  rw = XMdcResize(sGbc.w);
-  rh = XMdcResize(sGbc.h);
+  rw = (signed)XMdcResize(sGbc.w);
+  rh = (signed)XMdcResize(sGbc.h);
 
-  sGbc.img8 = MdcGetDisplBIT8_U(my.fi,sGbc.i);
+  MdcFree(sGbc.img8);
+
+  sGbc.img8 = MdcGetDisplayImage(my.fi,sGbc.i);
   if (sGbc.img8 == NULL) {
     XMdcDisplayErr("Couldn't alloc byte buffer");
     return;
   }
 
-  im = XMdcBuildGdkPixbuf(sGbc.img8,sGbc.w,sGbc.h,modtmp.vgbc,MDC_YES);
+  im = XMdcBuildGdkPixbuf(sGbc.img8,sGbc.w,sGbc.h,sGbc.t,modtmp.vgbc,MDC_YES);
   if (im == NULL) {
     MdcFree(sGbc.img8);
     XMdcDisplayErr("Couldn't create GdkPixbuf");
@@ -342,9 +361,7 @@ void XMdcColGbcCorrectSel(GtkWidget *widget, Uint32 nr)
 
     gdk_pixbuf_unref(sGbc.im);
 
-    sGbc.im=XMdcBuildGdkPixbuf(sGbc.img8,sGbc.w, sGbc.h,sGbc.mod.vgbc,MDC_YES);
-    if (sGbc.im == NULL) 
-      XMdcDisplayFatalErr(MDC_BAD_ALLOC,"Failure to create GdkPixbuf");
+    sGbc.im = im;
 
     gdk_window_clear(sGbc.area->window);
 
@@ -362,5 +379,6 @@ void XMdcColGbcCorrectSel(GtkWidget *widget, Uint32 nr)
   gtk_window_set_policy(GTK_WINDOW(wgbc), FALSE, FALSE, TRUE);
 
   XMdcShowWidget(wgbc);
+
 }
 
