@@ -7,19 +7,27 @@
  *                                                                         *
  * project      : (X)MedCon by Erik Nolf                                   *
  *                                                                         *
- * Functions    : XMdcRemovePreviousColorMap()    - Hide previous colormap *
- *                XMdcApplyNewColorMap()          - Apply new selected map *
- *                XMdcColorMapSelCallbackApply()  - Apply   callback       *
- *                XMdcColorMapSel()               - Select map & colors    *
- *                XMdcColorMapCallbackExpose()    - Expose  callback       *
- *                XMdcBuildColorMap()             - Build the colormap     *
+ * Functions    : XMdcRemovePreviousColorMap()   - Hide previous colormap  *
+ *                XMdcApplyNewColorMap()         - Apply new selected map  *
+ *                XMdcColorMapCallbackClicked()  - Clicked colormap        *
+ *                XMdcColorMapSelCallbackApply() - Apply   callback        *
+ *                XMdcColorMapSel()              - Select map & colors     *
+ *                XMdcColorMapCallbackExpose()   - Expose  callback        *
+ *                XMdcApplyMapPlace()            - Apply new placement     *
+ *                XMdcMapPlaceSel()              - Select map placement    *
+ *                XMdcMapPlaceSelCallbackApply() - Apply place selection   *
+ *                XMdcBuildColorMap()            - Build the colormap      *
+ *                XMdcLoadLUT()                  - Load the LUT file       * 
+ *                XMdcChangeLUT()                - Load another LUT file   *
+ *                XMdcMapNotAllowed()            - Not allowed on colorfile*
+ *                                                                         *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xcolmap.c,v 1.1.1.1 2000/10/28 16:51:39 enlf Exp $
+/* $Id: xcolmap.c,v 1.24 2004/01/04 22:36:17 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -40,6 +48,21 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
+
 #include "xmedcon.h"
 
 /****************************************************************************
@@ -47,6 +70,9 @@
 ****************************************************************************/
 
 static GtkWidget *wcolor=NULL;
+static GtkWidget *wplace=NULL;
+
+Uint8 XMDC_CMAP_PLACE = MDC_RIGHT;
 
 /****************************************************************************
                             F U N C T I O N S
@@ -59,6 +85,7 @@ void XMdcRemovePreviousColorMap(void)
 
 void XMdcApplyNewColorMap(int map)
 {
+
   gtk_widget_set_sensitive(my.viewwindow,FALSE);
 
   XMdcRemovePreviousColorMap();
@@ -70,36 +97,53 @@ void XMdcApplyNewColorMap(int map)
   XMdcBuildCurrentImages();
 
   gtk_widget_set_sensitive(my.viewwindow,TRUE);
+
+}
+
+gboolean XMdcColorMapCallbackClicked(GtkWidget *widget, GdkEventButton *button, gpointer data)
+{
+  if (button->button == 1) {
+    /* select color map */
+    XMdcColorMapSel();
+  }
+
+  if (button->button == 2) {
+    /* select color map */
+    XMdcColorMapSel();
+  }
+
+  if (button->button == 3) {
+    /* placement color map */
+    XMdcMapPlaceSel();
+  }
+
+  return(TRUE);
 }
 
 void XMdcColorMapSelCallbackApply (GtkWidget *widget, gpointer data)
 {
-  guint map=MDC_MAP_GRAY, invert=MDC_CMAP_INVERS;
+  gint map=MDC_MAP_GRAY;
 
- 
-  MdcDebugPrint("DEBUG: colormap ");
+  if (XMdcMapNotAllowed() == TRUE) return;
+
+  MdcDebugPrint("colormap type: "); 
   if (GTK_TOGGLE_BUTTON(sColormapSelection.Gray)->active)             {
-    MdcDebugPrint("gray normal");        map = MDC_MAP_GRAY;
-    sColormapSelection.Inverted=MDC_NO;
-  }else if (GTK_TOGGLE_BUTTON(sColormapSelection.GrayInvers)->active) {
-    MdcDebugPrint("gray invers"); map = MDC_MAP_GRAY; 
-    sColormapSelection.Inverted=MDC_YES;
+    MdcDebugPrint("\tgray normal"); map = MDC_MAP_GRAY;
+  }else if (GTK_TOGGLE_BUTTON(sColormapSelection.Inverted)->active) {
+    MdcDebugPrint("\tgray invers"); map = MDC_MAP_INVERTED; 
   }else if (GTK_TOGGLE_BUTTON(sColormapSelection.Rainbow)->active)    {
-    MdcDebugPrint("rainbow");     map = MDC_MAP_RAINBOW;
+    MdcDebugPrint("\trainbow");     map = MDC_MAP_RAINBOW;
   }else if (GTK_TOGGLE_BUTTON(sColormapSelection.Combined)->active)   {
-    MdcDebugPrint("combined");    map = MDC_MAP_COMBINED;
+    MdcDebugPrint("\tcombined");    map = MDC_MAP_COMBINED;
   }else if (GTK_TOGGLE_BUTTON(sColormapSelection.Hotmetal)->active)   {
-    MdcDebugPrint("hotmetal");    map = MDC_MAP_HOTMETAL;
+    MdcDebugPrint("\thotmetal");    map = MDC_MAP_HOTMETAL;
   }else if (GTK_TOGGLE_BUTTON(sColormapSelection.Loaded)->active)     {
-    MdcDebugPrint("loaded");      map = MDC_MAP_LOADED;
+    MdcDebugPrint("\tloaded");      map = MDC_MAP_LOADED;
   }
 
-  MdcDebugPrint(" selected\n");
-
   if (map == MDC_MAP_LOADED) { XMdcLutSelOpen(NULL,NULL); return; }
 
-  if (map!=sColormapSelection.CurMap || invert!=sColormapSelection.Inverted ) {
-    MDC_CMAP_INVERS = sColormapSelection.Inverted; 
+  if (map != sColormapSelection.CurMap) {
     sColormapSelection.CurMap = map; MDC_COLOR_MAP = map;
     if (XMDC_FILE_OPEN == MDC_YES) XMdcApplyNewColorMap(map);
     else XMdcColorMapReset(map);
@@ -119,6 +163,9 @@ gboolean XMdcColorMapSel(void)
   GtkWidget *separator;
   GSList *group;
 
+
+  if (XMdcMapNotAllowed() == TRUE) return(TRUE);
+
   if (wcolor == NULL) {
 
     wcolor = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -158,8 +205,7 @@ gboolean XMdcColorMapSel(void)
     /* create radiobuttons */
     button = gtk_radio_button_new_with_label (NULL, "Gray Normal");
     gtk_box_pack_start (GTK_BOX (box4), button, TRUE, TRUE, 0);
-    if (sColormapSelection.CurMap   == MDC_MAP_GRAY &&
-        sColormapSelection.Inverted == MDC_NO)
+    if (sColormapSelection.CurMap   == MDC_MAP_GRAY)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
     gtk_widget_show (button);
     sColormapSelection.Gray = button;
@@ -167,11 +213,10 @@ gboolean XMdcColorMapSel(void)
     group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
     button = gtk_radio_button_new_with_label(group, "Gray Invers");
     gtk_box_pack_start (GTK_BOX (box4), button, TRUE, TRUE, 0);
-    if (sColormapSelection.CurMap   == MDC_MAP_GRAY &&
-        sColormapSelection.Inverted == MDC_YES)
+    if (sColormapSelection.CurMap   == MDC_MAP_INVERTED)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
     gtk_widget_show (button);
-    sColormapSelection.GrayInvers = button;
+    sColormapSelection.Inverted = button;
 
     group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
     button = gtk_radio_button_new_with_label(group, "Rainbow");
@@ -236,7 +281,7 @@ gboolean XMdcColorMapSel(void)
     gtk_widget_hide(wcolor);
 
     b1 = sColormapSelection.Gray;
-    b2 = sColormapSelection.GrayInvers;
+    b2 = sColormapSelection.Inverted;
     b3 = sColormapSelection.Rainbow;
     b4 = sColormapSelection.Combined;
     b5 = sColormapSelection.Hotmetal;
@@ -249,11 +294,10 @@ gboolean XMdcColorMapSel(void)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b6),FALSE);
     switch (sColormapSelection.CurMap) {
       case MDC_MAP_GRAY    :
-          if (sColormapSelection.Inverted == MDC_NO) {
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1),TRUE);
-          }else{
-            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2),TRUE);
-          }
+          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1),TRUE);
+          break;
+      case MDC_MAP_INVERTED:
+          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2),TRUE);
           break;
       case MDC_MAP_RAINBOW :
           gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b3),TRUE);
@@ -293,15 +337,166 @@ gboolean XMdcColorMapCallbackExpose(GtkWidget *widget,GdkEventExpose *event,gpoi
  
 }
 
+void XMdcApplyMapPlace(int place)
+{
+
+  GtkWidget *parent=NULL;
+
+  if (XMDC_FILE_OPEN == MDC_YES) {
+
+    parent = my.cmapbox->parent;
+
+    switch (place) {
+
+      case MDC_LEFT :
+          gtk_box_reorder_child(GTK_BOX(parent),my.cmapbox,0);
+          gtk_box_reorder_child(GTK_BOX(parent),my.imgsbox,2);
+          break;
+
+      case MDC_RIGHT:
+          gtk_box_reorder_child(GTK_BOX(parent),my.cmapbox,2);
+          gtk_box_reorder_child(GTK_BOX(parent),my.imgsbox,0);
+          break;
+    } 
+  }
+}
+
+void XMdcMapPlaceSelCallbackApply (GtkWidget *widget, gpointer data)
+{
+  MdcDebugPrint("colormap location: ");
+  if (GTK_TOGGLE_BUTTON(sMapPlaceSelection.Left)->active)             {
+    MdcDebugPrint("\tleft");  XMDC_CMAP_PLACE = MDC_LEFT;
+  }else if (GTK_TOGGLE_BUTTON(sMapPlaceSelection.Right)->active) {
+    MdcDebugPrint("\tright"); XMDC_CMAP_PLACE = MDC_RIGHT;
+  }
+
+  XMdcApplyMapPlace(XMDC_CMAP_PLACE);
+
+}
+
+gboolean XMdcMapPlaceSel(void)
+{
+  GtkWidget *box1;
+  GtkWidget *box2;
+  GtkWidget *box3;
+  GtkWidget *box4;
+  GtkWidget *frame;
+  GtkWidget *button;
+  GtkWidget *separator;
+  GSList *group;
+
+  if (wplace == NULL) {
+
+    wplace = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+
+    gtk_signal_connect(GTK_OBJECT(wplace),"destroy",
+                       GTK_SIGNAL_FUNC(XMdcMedconQuit),NULL);
+    gtk_signal_connect(GTK_OBJECT(wplace),"delete_event",
+                       GTK_SIGNAL_FUNC(XMdcHandlerToHide),NULL);
+
+
+    gtk_window_set_title(GTK_WINDOW(wplace),"Placement Selection");
+    gtk_container_set_border_width (GTK_CONTAINER (wplace), 0);
+
+    box1 = gtk_vbox_new (FALSE, 0);
+    gtk_container_add (GTK_CONTAINER (wplace), box1);
+    gtk_widget_show(box1);
+
+    /* create upper box - placement */
+    box2 = gtk_vbox_new (FALSE, 5);
+    gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER(box2), 5);
+    gtk_widget_show(box2);
+
+    box3 = gtk_hbox_new (FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(box2), box3, TRUE, TRUE, 0);
+    gtk_widget_show(box3);
+
+    /* create placement frame */
+    frame = gtk_frame_new("Placement");
+    gtk_box_pack_start(GTK_BOX (box3), frame, TRUE, TRUE, 0);
+    gtk_widget_show(frame);
+
+    box4 = gtk_vbox_new(FALSE, 0);
+    gtk_container_add(GTK_CONTAINER(frame), box4);
+    gtk_container_set_border_width(GTK_CONTAINER(box4), 5);
+    gtk_widget_show(box4);
+    /* create radiobuttons */
+    button = gtk_radio_button_new_with_label (NULL, "Left  side");
+    gtk_box_pack_start (GTK_BOX (box4), button, TRUE, TRUE, 0);
+    if (XMDC_CMAP_PLACE == MDC_LEFT)
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+    gtk_widget_show (button);
+    sMapPlaceSelection.Left = button;
+
+    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
+    button = gtk_radio_button_new_with_label(group, "Right side");
+    gtk_box_pack_start (GTK_BOX (box4), button, TRUE, TRUE, 0);
+    if (XMDC_CMAP_PLACE == MDC_RIGHT)
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+    gtk_widget_show (button);
+    sMapPlaceSelection.Right = button;
+
+    /* create horizontal separator */
+    separator = gtk_hseparator_new ();
+    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);
+    gtk_widget_show (separator);
+
+    /* create bottom button box */
+    box2 = gtk_hbox_new (FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(box1), box2, TRUE, TRUE, 2);
+    gtk_widget_show(box2);
+
+    button = gtk_button_new_with_label("Apply");
+    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 2);
+    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+                       GTK_SIGNAL_FUNC(gtk_widget_hide),GTK_OBJECT(wplace));
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                       GTK_SIGNAL_FUNC(XMdcMapPlaceSelCallbackApply), NULL);
+    gtk_widget_show(button);
+
+    button = gtk_button_new_with_label ("Cancel");
+    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 2);
+    gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
+        GTK_SIGNAL_FUNC(gtk_widget_hide),GTK_OBJECT(wplace));
+    gtk_widget_show(button);
+
+  }else{
+     /* set buttons to appropriate state */
+    GtkWidget *b1, *b2;
+
+    gtk_widget_hide(wplace);
+
+    b1 = sMapPlaceSelection.Left;
+    b2 = sMapPlaceSelection.Right;
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1),FALSE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2),FALSE);
+    switch (XMDC_CMAP_PLACE) {
+      case MDC_LEFT :
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1),TRUE);
+          break;
+      case MDC_RIGHT:
+          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2),TRUE);
+          break;
+    }
+
+  }
+
+  XMdcShowWidget(wplace);
+
+  return(TRUE);
+
+
+}
 
 void XMdcBuildColorMap(void)
 {
   Uint8 *imgRGB, rr, gg, bb;
   Uint32 pix, i, r, c;
 
-  MdcDebugPrint("DEBUG: Build colormap ...\n");
+  MdcDebugPrint("Building colormap ...");
 
-  imgRGB = (Uint8 *)malloc(my.cmap_w * my.cmap_h * 3);
+  imgRGB = (Uint8 *)malloc((unsigned)(my.cmap_w * my.cmap_h * 3));
 
   if (imgRGB == NULL)
     XMdcDisplayFatalErr(MDC_BAD_ALLOC,"Couldn't alloc colormap imgRGB");
@@ -333,7 +528,7 @@ void XMdcBuildColorMap(void)
     gtk_container_add(GTK_CONTAINER(my.cmapbox), my.cmap);
 
     gtk_signal_connect(GTK_OBJECT(my.cmap),"button_press_event",
-                       GTK_SIGNAL_FUNC(XMdcColorMapSel), NULL);
+                       GTK_SIGNAL_FUNC(XMdcColorMapCallbackClicked), NULL);
 
     gtk_signal_connect(GTK_OBJECT(my.cmap),"expose_event",
                        GTK_SIGNAL_FUNC(XMdcColorMapCallbackExpose), NULL);
@@ -342,7 +537,90 @@ void XMdcBuildColorMap(void)
   
   gtk_drawing_area_size(GTK_DRAWING_AREA(my.cmap), my.cmap_w, my.cmap_h);
 
-  gtk_widget_show(my.cmap); 
+  if (my.fi->type != COLRGB) gtk_widget_show(my.cmap); 
+
+}
+
+int XMdcLoadLUT(gchar *lutname)
+{
+  if (MdcLoadLUT(lutname) == MDC_YES) {
+
+    sColormapSelection.CurMap = MDC_MAP_LOADED;
+    MDC_COLOR_MAP = MDC_MAP_LOADED;
+
+    if (XMDC_FILE_OPEN == MDC_YES) XMdcApplyNewColorMap(MDC_COLOR_MAP);
+
+  }else{
+
+    /* XMdcDisplayWarn("Couldn't load specified LUT file");*/ return(MDC_NO);
+
+  }
+
+  return(MDC_YES);
+
+}
+
+gboolean XMdcChangeLUT(GtkWidget *spinner, gpointer data)
+{
+
+  int nr;
+  gchar lutname[10];
+  GtkSpinButton *spin = GTK_SPIN_BUTTON(spinner);
+
+  if (XMdcMapNotAllowed() == MDC_YES) return(TRUE);
+
+  nr = gtk_spin_button_get_value_as_int(spin);
+
+  if (nr == sColormapSelection.Nr) return(TRUE);
+
+  /* make appropriate path name */
+  sprintf(lutname,"ct%03d.lut",nr);
+  if (XMEDCONLUT != NULL) { 
+    strncpy(xmdcstr,XMEDCONLUT,MDC_1KB_OFFSET); xmdcstr[strlen(xmdcstr)]='\0';
+#ifdef MDC_DOS
+     if (xmdcstr[strlen(xmdcstr)-1] != '\\' ) strcat(xmdcstr,"\\"); 
+#else
+     if (xmdcstr[strlen(xmdcstr)-1] != '/' ) strcat(xmdcstr,"/");
+#endif
+  }else{
+    /* installation dir */
+    strncpy(xmdcstr,XMDCLUT,MDC_1KB_OFFSET);    xmdcstr[strlen(xmdcstr)]='\0';
+  }
+
+  sprintf(lutname,"ct%03d.lut",nr); strcat(xmdcstr,lutname);
+
+
+  if (XMdcLoadLUT(xmdcstr) == MDC_YES) {
+    /* set to new LUT number */
+    sColormapSelection.Nr = nr;
+  }
+
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner)
+                            ,(gfloat)sColormapSelection.Nr);
+
+  return(TRUE);
+
+}
+
+/* prevent gray maps on colored images */
+gboolean XMdcMapNotAllowed(void)
+{
+
+  /* no file opened, can select grayscale colormap */
+  if (XMDC_FILE_OPEN == MDC_NO) return(FALSE);
+
+  /* no grayscale colormap selection for colored files */
+  if ((MDC_MAKE_GRAY == MDC_NO) && (my.fi->map == MDC_MAP_PRESENT)) {
+
+    XMdcDisplayWarn("Grayscale tables not available for colored images.\n"\
+                    "Otherwise force remap to grayscale in\n"\
+                    "Options || MedCon || Slices");
+
+    return(TRUE);
+  }
+
+  /* default allow grayscale colormap selection */
+  return(FALSE);
 
 }
 
