@@ -8,11 +8,11 @@
  * project      : (X)MedCon by Erik Nolf                                   *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xdefs.h,v 1.1.1.1 2000/10/28 16:51:41 enlf Exp $
+/* $Id: xdefs.h,v 1.52 2004/02/13 23:41:50 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -35,12 +35,20 @@
                               D E F I N E S 
 ****************************************************************************/
 
-#define XMDC_NORMAL  1           /* file types in XMedCon         */         
-#define XMDC_RAW     2           /* unsaved file                  */ 
-#define XMDC_EXTRACT 3           /* unsaved file                  */
-#define XMDC_SAVED   4           /* saved   file                  */
+/* file types */                 /* unsaved status > 128*/
+#define XMDC_RAW     255
+#define XMDC_PREDEF  254
+#define XMDC_EXTRACT 253
+#define XMDC_RESLICE 252
+#define XMDC_TRANSF  251
+#define XMDC_EDITFI  250
 
-#define XMDC_FREE_BORDER 100
+#define XMDC_UNSAVED 128
+                                 /* saved   status < 128 */
+#define XMDC_SAVED     2
+#define XMDC_NORMAL    1
+
+#define XMDC_FREE_BORDER 125
 #define XMDC_COLORMAP_WIDTH  25
 #define XMDC_COLORMAP_HEIGHT 48
 
@@ -61,8 +69,14 @@
 #define XMDC_RESIZE_HALF     -(XMDC_ZOOM_FACTOR)   /* divide   by 2 */
 #define XMDC_RESIZE_DOUBLE    (XMDC_ZOOM_FACTOR)   /* multiply by 2 */
 
+#define XMDC_PAGES_FRAME_BY_FRAME 0                   /* frame by frame */
+#define XMDC_PAGES_SLICE_BY_SLICE 1                   /* slice by slice */
+#define XMDC_PAGES_SCREEN_FULL    2                   /* screen full    */ 
+
 #define XMDC_DEFAULT_FRMT  MDC_FRMT_RAW  /* default save format (enabled!?) */
 
+#define XMDC_MAX_LOADABLE_LUTS  125      /* maximum external LUTs available */
+
 #ifdef _WIN32
 #define MDC_USE_SIGNAL_BLOCKER 1 
 #endif
@@ -75,6 +89,7 @@ typedef struct SignalBlocker_t{
 typedef struct OptionsMedConStruct_t {
         GtkWidget *PixPositives;
         GtkWidget *PixNegatives;
+        GtkWidget *PixNoQuant;
         GtkWidget *PixQuantify;
         GtkWidget *PixCalibrate;
         GtkWidget *PixTypeNONE;
@@ -83,16 +98,46 @@ typedef struct OptionsMedConStruct_t {
         GtkWidget *BitsUsed12;
         GtkWidget *FileTypeLITTLE;
         GtkWidget *FileTypeBIG;
+        GtkWidget *FlipHoriz;
+        GtkWidget *FlipVert;
+        GtkWidget *SortReverse;
+        GtkWidget *SortCine;
+        GtkWidget *SortCineApply;
+        GtkWidget *SortCineUndo;
+        GtkWidget *MakeSqrNo;
+        GtkWidget *MakeSqr1;
+        GtkWidget *MakeSqr2;
         GtkWidget *NormOverFrames;
         GtkWidget *NormOverAll;
         GtkWidget *FallbackNONE;
         GtkWidget *FallbackANLZ;
+        GtkWidget *FallbackCONC;
         GtkWidget *FallbackECAT;
         GtkWidget *FallbackDICM;
-        GtkWidget *AnlzRev;
+        GtkWidget *SplitNone;
+        GtkWidget *SplitFrames;
+        GtkWidget *SplitSlices;
+        GtkWidget *ColorModeIndexed;
+        GtkWidget *ColorMakeGray;
+        GtkWidget *ColorDither;
+        GtkWidget *NameAlias;
+        GtkWidget *NameNoPrefix;
+        GtkWidget *DicmMosaicEnabled;
+        GtkWidget *DicmMosaicForced;
+        GtkWidget *DicmMosaicWidth;
+        GtkWidget *DicmMosaicHeight;
+        GtkWidget *DicmMosaicNumber;
+        GtkWidget *DicmMosaicDoInterl;
+        GtkWidget *DicmMosaicFixVoxel;
+        GtkWidget *DicmTrueGap;
+        GtkWidget *DicmContrast;
+        GtkWidget *DicmWriteImplicit;
+        GtkWidget *DicmWriteNoMeta;
         GtkWidget *AnlzSPM;
-        GtkWidget *GifMakeGray;
-        GtkWidget *GifSaveWithDialog;
+        GtkWidget *IntfSkip1;
+        GtkWidget *IntfNoPath;
+        GtkWidget *EcatSortAnatom;
+        GtkWidget *EcatSortByFrame;
 }OptionsMedConStruct;
 
 typedef struct MyMainStruct_t {
@@ -100,6 +145,7 @@ typedef struct MyMainStruct_t {
     GtkWidget *viewwindow;
     GtkWidget *viewbox;
     GtkWidget *pagemenu;
+    GtkWidget *imgsbox;
     GtkWidget *imgstable;
     GtkWidget *cmapbox;
     GtkWidget **image;
@@ -120,16 +166,20 @@ typedef struct MyMainStruct_t {
 
 
 typedef struct ColormapSelectionStruct_t {
+   gint      Nr;
    gint      CurMap;
-   gint      Inverted;
    GtkWidget *Gray;
-   GtkWidget *GrayInvers;
+   GtkWidget *Inverted;
    GtkWidget *Rainbow;
    GtkWidget *Combined;
    GtkWidget *Hotmetal;
    GtkWidget *Loaded;
 }ColormapSelectionStruct;
 
+typedef struct MapPlaceSelectionStruct_t {
+   GtkWidget *Right;
+   GtkWidget *Left;
+}MapPlaceSelectionStruct;
 
 typedef struct LabelSelectionStruct_t {
    gint CurState;
@@ -185,6 +235,7 @@ typedef struct RawReadSelectionStruct_t {
    GtkWidget *typeBIT32_S, *typeBIT32_U;
    GtkWidget *typeBIT64_S, *typeBIT64_U;
    GtkWidget *typeFLT32,   *typeFLT64;
+   GtkWidget *typeCOLRGB;
    Uint32 ImgCounter;
 }RawReadSelectionStruct;
 
@@ -196,6 +247,13 @@ typedef struct ResizeSelectionStruct_t {
    GtkWidget *Double;
 }ResizeSelectionStruct;
 
+typedef struct PagesSelectionStruct_t{
+   Int8 CurType;
+   GtkWidget *FrameByFrame;
+   GtkWidget *SliceBySlice;
+   GtkWidget *ScreenFull;
+}PagesSelectionStruct;
+
 typedef struct SliderValueStruct_t {
   GtkObject  *adj;
   GtkWidget  *range;
@@ -215,18 +273,36 @@ typedef struct ColGbcCorrectStruct_t {
   GdkPixbuf *im;
   ColorModifier mod;
   Uint32 i, nr, w, h, rw, rh;
+  Int16 t;
   Uint8 *img8, vgbc[256];
 }ColGbcCorrectStruct;
 
+typedef struct EditFileInfoStruct_t {
+  int CurModality;
+  GtkWidget *PatSliceOrient[MDC_MAX_ORIENT];
+  GtkWidget *PixelSize;
+  GtkWidget *SliceWidth;
+  GtkWidget *SliceSpacing;
+  GtkWidget *FrameDuration;
+  GtkWidget *NrDimPlanes;
+  GtkWidget *NrDimFrames;
+  GtkWidget *NrDimGates;
+  GtkWidget *NrDimBeds;
+  GtkWidget *NrDimWindows;
+  GtkWidget *Reconstructed;
+  GtkWidget *Planar;
+  GtkWidget *ModalityNM;
+  GtkWidget *ModalityPT;
+  GtkWidget *AcquisitionType[MDC_MAX_ACQUISITIONS];
+}EditFileInfoStruct;
+
 extern Uint8 XMDC_FILE_OPEN;
 extern Uint8 XMDC_FILE_TYPE;
 
 extern Uint8 XMDC_IMAGE_BORDER;
+extern Uint8 XMDC_CMAP_PLACE;
 
 extern Uint8 XMDC_DOBAR;
-extern gfloat pvalue;
-extern GtkWidget *pbarwindow;
-extern GtkWidget *pbar;
 
 extern GdkColor Red;
 extern GdkColor Green;
@@ -241,19 +317,23 @@ extern GdkFont *sfixed;
 extern MyMainStruct            my;
 extern OptionsMedConStruct     sOptionsMedCon;
 extern ColormapSelectionStruct sColormapSelection;
+extern MapPlaceSelectionStruct sMapPlaceSelection;
 extern LabelSelectionStruct    sLabelSelection;
 extern RenderSelectionStruct   sRenderSelection;
 extern ExtractSelectionStruct  sExtractSelection;
 extern RawReadSelectionStruct  sRawReadSelection;
 extern ResizeSelectionStruct   sResizeSelection;
-
+extern PagesSelectionStruct    sPagesSelection;
 extern ColGbcCorrectStruct     sGbc;
+extern EditFileInfoStruct      sEditFI;
 
-extern char labelname[25];
-extern Int8 pages_to_frames;
+extern char labelindex[25];
+extern char labeltimes[50];
 extern Uint32 write_counter;
 
 extern char xmdcstr[MDC_2KB_OFFSET];
+extern char *XMEDCONLUT;
+extern char *XMEDCONRPI;
 
 #endif
 
