@@ -13,19 +13,24 @@
  *                XMdcFileSelOpen()                 - Open file selection  *
  *                XMdcFileSelSaveCreateFormatMenu() - Create Format menu   *
  *                XMdcFileSelSaveCreateDefaultName()- Create Default name  *
+ *                XMdcFileSelSaveCallbackAlias()    - Get alias   filename *
  *                XMdcFileSelSaveCallbackDefault()  - Get default filename *
  *                XMdcFileSelSaveCallbackCancel()   - Save Cancel callback *
  *                XMdcFileSelSaveCallbackOk()       - Save Ok     callback *
  *                XMdcFileSelSave()                 - Save file selection  *
  *                XMdcLutSelOpenCallbackOk()        - LUT Open Ok callback *
- *                XMdcLutSelOpen()                  - LUT open file        * 
+ *                XMdcLutSelOpen()                  - LUT open file        *
+ *                XMdcRawPredefSelSaveCallbackOk()  - Raw Predef Save Ok   *
+ *                XMdcRawPredefSelSave()            - Raw Predef Save      *
+ *                XMdcRawPredefSelOpenCallbackOk()  - Raw Predef Load Ok   *
+ *                XMdcRawPredefSelOpen()            - Raw Predef Load      * 
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xfilesel.c,v 1.1.1.1 2000/10/28 16:51:46 enlf Exp $
+/* $Id: xfilesel.c,v 1.26 2004/05/09 22:48:03 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -45,15 +50,29 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
+
 #include "xmedcon.h"
 
 /****************************************************************************
                               D E F I N E S
 ****************************************************************************/
 
-static Int8  format_to_use[MDC_MAX_FRMTS], mitem_nr[MDC_MAX_FRMTS];
+static guint format_to_use[MDC_MAX_FRMTS], mitem_nr[MDC_MAX_FRMTS];
 static guint OPEN_TYPE=XMDC_NORMAL;
 static GtkWidget *ofilew=NULL, *sfilew=NULL, *olutw=NULL, *formatmenu=NULL;
+static GtkWidget *opredefw=NULL, *spredefw=NULL;
+static GtkWidget *default_name=NULL;
 
 /****************************************************************************
                             F U N C T I O N S
@@ -66,7 +85,11 @@ void XMdcFileSelOpenCallbackOk(GtkWidget *w, GtkWidget *fs)
 
     if (OPEN_TYPE == XMDC_RAW) {
 
-      XMdcInteractiveReadBegin(fs);
+      XMdcRawReadInteractive(fs);
+
+    }else if(OPEN_TYPE == XMDC_PREDEF) {
+
+      XMdcRawReadPredef(fs);
 
     }else{
 
@@ -110,7 +133,9 @@ void XMdcFileSelOpen(GtkWidget *widget, guint otype)
     }
 
     if (OPEN_TYPE == XMDC_RAW) {
-      gtk_window_set_title(GTK_WINDOW(ofilew),"Open RAW File");
+      gtk_window_set_title(GTK_WINDOW(ofilew),"Open RAW File (interactive)");
+    }else if (OPEN_TYPE == XMDC_PREDEF) {
+      gtk_window_set_title(GTK_WINDOW(ofilew),"Open RAW File (predefined)");
     }else{
       gtk_window_set_title(GTK_WINDOW(ofilew),"Open File");
     }
@@ -121,24 +146,33 @@ void XMdcFileSelOpen(GtkWidget *widget, guint otype)
 				     "<unknown>");
     XMdcShowWidget(ofilew);
 
-    if (XMDC_FILE_TYPE == XMDC_EXTRACT)
-      XMdcDisplayWarn("Extracted images were not saved!");
-    if (XMDC_FILE_TYPE == XMDC_RAW)
-      XMdcDisplayWarn("RAW file was not saved!");
-
+    switch (XMDC_FILE_TYPE) {
+      case XMDC_RAW    : XMdcDisplayWarn("RAW file was not saved");
+          break;
+      case XMDC_PREDEF : XMdcDisplayWarn("RAW file was not saved");
+          break;
+      case XMDC_EXTRACT: XMdcDisplayWarn("Extracted images were not saved");
+          break;
+      case XMDC_RESLICE: XMdcDisplayWarn("Resliced images not saved");
+          break;
+      case XMDC_TRANSF : XMdcDisplayWarn("Transformed images not saved");
+          break;
+      case XMDC_EDITFI : XMdcDisplayWarn("Changed FileInfo not saved");
+          break;
+    }
 }
 
 void XMdcFileSelSaveCreateFormatMenu(GtkWidget *fs, guint format)
 {
     GtkWidget *menu=NULL;
     GtkWidget *menuitem; 
-    Int8 item=0, selected=0;
+    guint item=0, selected=0;
     Uint8 LOOK=MDC_YES;
 
     formatmenu = gtk_option_menu_new();
     menu = gtk_menu_new();
 
-    memset(format_to_use,MDC_FRMT_BAD,MDC_MAX_FRMTS);
+    memset(format_to_use,MDC_FRMT_NONE,MDC_MAX_FRMTS);
     memset(mitem_nr,-1,MDC_MAX_FRMTS);
 
 /* format Raw Binary */
@@ -205,18 +239,49 @@ void XMdcFileSelSaveCreateFormatMenu(GtkWidget *fs, guint format)
     gtk_menu_append(GTK_MENU(menu),menuitem);
     gtk_widget_show(menuitem);
 #endif
-#if MDC_INCLUDE_ECAT
-    if (LOOK && (format == MDC_FRMT_ECAT)) {
-      XMDC_WRITE_FRMT = MDC_FRMT_ECAT; LOOK = MDC_NO; selected = item;
+#if MDC_INCLUDE_CONC
+    if (LOOK && (format == MDC_FRMT_CONC)) {
+      XMDC_WRITE_FRMT = MDC_FRMT_CONC; LOOK = MDC_NO; selected = item;
     }
-    format_to_use[MDC_FRMT_ECAT]=MDC_FRMT_ECAT;
-    mitem_nr[MDC_FRMT_ECAT] = item++;
+    format_to_use[MDC_FRMT_CONC]=MDC_FRMT_CONC;
+    mitem_nr[MDC_FRMT_CONC] = item++;
 
-    menuitem = gtk_menu_item_new_with_label(FrmtString[MDC_FRMT_ECAT]);
+    menuitem = gtk_menu_item_new_with_label(FrmtString[MDC_FRMT_CONC]);
 
     gtk_signal_connect( GTK_OBJECT(menuitem), "activate",
                         GTK_SIGNAL_FUNC(XMdcFileSelSaveCallbackFormatMenu),
-                        &format_to_use[MDC_FRMT_ECAT]);
+                        &format_to_use[MDC_FRMT_CONC]);
+
+    gtk_menu_append(GTK_MENU(menu),menuitem);
+    gtk_widget_show(menuitem);
+#endif
+#if MDC_INCLUDE_DICM
+    if (LOOK && (format == MDC_FRMT_DICM)) {
+      XMDC_WRITE_FRMT = MDC_FRMT_DICM; LOOK = MDC_NO; selected = item;
+    }
+    format_to_use[MDC_FRMT_DICM]=MDC_FRMT_DICM;
+    mitem_nr[MDC_FRMT_DICM] = item++;
+
+    menuitem = gtk_menu_item_new_with_label(FrmtString[MDC_FRMT_DICM]);
+    gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+                       GTK_SIGNAL_FUNC(XMdcFileSelSaveCallbackFormatMenu),
+                       &format_to_use[MDC_FRMT_DICM]);
+
+    gtk_menu_append(GTK_MENU(menu),menuitem);
+    gtk_widget_show(menuitem);
+#endif
+#if MDC_INCLUDE_ECAT
+    if (LOOK && (format == MDC_FRMT_ECAT6)) {
+      XMDC_WRITE_FRMT = MDC_FRMT_ECAT6; LOOK = MDC_NO; selected = item;
+    }
+    format_to_use[MDC_FRMT_ECAT6]=MDC_FRMT_ECAT6;
+    mitem_nr[MDC_FRMT_ECAT6] = item++;
+
+    menuitem = gtk_menu_item_new_with_label(FrmtString[MDC_FRMT_ECAT6]);
+
+    gtk_signal_connect( GTK_OBJECT(menuitem), "activate",
+                        GTK_SIGNAL_FUNC(XMdcFileSelSaveCallbackFormatMenu),
+                        &format_to_use[MDC_FRMT_ECAT6]);
 
     gtk_menu_append(GTK_MENU(menu),menuitem);
     gtk_widget_show(menuitem);
@@ -268,22 +333,23 @@ void XMdcFileSelSaveCreateFormatMenu(GtkWidget *fs, guint format)
     gtk_menu_append(GTK_MENU(menu),menuitem);
     gtk_widget_show(menuitem);
 #endif
-#if MDC_INCLUDE_DICM
-    if (LOOK && (format == MDC_FRMT_DICM)) {
-      XMDC_WRITE_FRMT = MDC_FRMT_DICM; LOOK = MDC_NO; selected = item;
+#if MDC_INCLUDE_PNG
+    if (LOOK && (format == MDC_FRMT_PNG)) {
+      XMDC_WRITE_FRMT = MDC_FRMT_PNG; LOOK = MDC_NO; selected = item;
     }
-    format_to_use[MDC_FRMT_DICM]=MDC_FRMT_DICM;
-    mitem_nr[MDC_FRMT_DICM] = item++;
+    format_to_use[MDC_FRMT_PNG]=MDC_FRMT_PNG;
+    mitem_nr[MDC_FRMT_PNG] = item++;
 
-    menuitem = gtk_menu_item_new_with_label(FrmtString[MDC_FRMT_DICM]);
+    menuitem = gtk_menu_item_new_with_label(FrmtString[MDC_FRMT_PNG]);
     gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
                        GTK_SIGNAL_FUNC(XMdcFileSelSaveCallbackFormatMenu),
-                       &format_to_use[MDC_FRMT_DICM]);
+                       &format_to_use[MDC_FRMT_PNG]);
 
     gtk_menu_append(GTK_MENU(menu),menuitem);
     gtk_widget_show(menuitem);
 #endif
 
+
     if (LOOK) XMdcDisplayFatalErr(MDC_BAD_CODE,"Unexpected format to save");
 
     XMdcFileSelSaveCreateDefaultName(fs);
@@ -299,23 +365,36 @@ void XMdcFileSelSaveCreateFormatMenu(GtkWidget *fs, guint format)
 void XMdcFileSelSaveCreateDefaultName(GtkWidget *fs)
 {
 
-  MdcDefaultName(XMDC_WRITE_FRMT,my.fi->ofname,my.fi->ifname);
+  MdcDefaultName(my.fi,XMDC_WRITE_FRMT,my.fi->ofname,my.fi->ifname);
 
   gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),my.fi->ofname);
  
 } 
   
 
-void XMdcFileSelSaveCallbackFormatMenu(GtkWidget *fs, Int8 *selected_format)
+void XMdcFileSelSaveCallbackFormatMenu(GtkWidget *fs, guint *selected_format)
 {
-   XMDC_WRITE_FRMT = *selected_format;
+   XMDC_WRITE_FRMT = (Int8)(*selected_format);
 
    XMdcFileSelSaveCreateDefaultName(sfilew);
 }
 
+void XMdcFileSelSaveCallbackAlias(GtkObject *fs, char *filename)
+{
+  Int8 prev = MDC_ALIAS_NAME;
+
+  MDC_ALIAS_NAME = MDC_YES;
+
+  MdcDefaultName(my.fi,XMDC_WRITE_FRMT,my.fi->ofname,my.fi->ifname);
+
+  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),my.fi->ofname);
+
+  MDC_ALIAS_NAME = prev;
+}
    
 void XMdcFileSelSaveCallbackDefault(GtkObject *fs, char *filename)
 {
+  MdcDefaultName(my.fi,XMDC_WRITE_FRMT,my.fi->ofname,my.fi->ifname);
   gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),my.fi->ofname);
 }
 
@@ -338,11 +417,10 @@ void XMdcFileSelSaveCallbackOk(GtkWidget *widget, GtkWidget *fs)
    MdcMergePath(my.fi->opath,my.fi->odir,my.fi->ofname);
 
    if (XMdcWriteFile(XMDC_WRITE_FRMT)) {
-     if ((XMDC_FILE_TYPE == XMDC_RAW) || (XMDC_FILE_TYPE == XMDC_EXTRACT))
-       XMDC_FILE_TYPE = XMDC_SAVED;
+     if (XMDC_FILE_TYPE >= XMDC_UNSAVED) XMDC_FILE_TYPE = XMDC_SAVED;
    }
 
-   XMdcEndProgressBar();
+   XMdcProgressBar(MDC_PROGRESS_END,0.,NULL);
 
 }
 
@@ -352,11 +430,13 @@ void XMdcFileSelSave(GtkWidget *widget, guint format)
 
    GtkWidget *menu=NULL;
    GtkWidget *active=NULL;
-   Int8 nr=mitem_nr[format];
-    
+   guint nr=mitem_nr[format];
+
+   nr = (format < MDC_MAX_FRMTS) ? mitem_nr[format] : 1;
+
    if (XMdcNoFileOpened()) return;
 
-   MdcPrefix(write_counter++);
+   MdcPrefix((signed)write_counter++);
 
    if (sfilew == NULL) {
 
@@ -389,12 +469,21 @@ void XMdcFileSelSave(GtkWidget *widget, guint format)
 
      XMdcFileSelSaveCreateFormatMenu(sfilew,format);
 
+     button = gtk_button_new_with_label("Alias Name");
+     gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+        GTK_SIGNAL_FUNC(XMdcFileSelSaveCallbackAlias), GTK_OBJECT(sfilew));
+     gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(sfilew)->action_area),
+                        button, TRUE, TRUE, 0);
+     gtk_widget_show(button);
+
      button = gtk_button_new_with_label("Default Name");
      gtk_signal_connect_object(GTK_OBJECT(button), "clicked", 
         GTK_SIGNAL_FUNC(XMdcFileSelSaveCallbackDefault), GTK_OBJECT(sfilew));
      gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(sfilew)->action_area),
-                        button, FALSE, FALSE, 0);
-     gtk_widget_show(button);
+                        button, TRUE, TRUE, 0);
+     gtk_widget_show(button); 
+
+     default_name = button;
 
    }else{
    
@@ -417,6 +506,9 @@ void XMdcFileSelSave(GtkWidget *widget, guint format)
 
   }
 
+  if (MDC_ALIAS_NAME == MDC_YES) gtk_widget_hide(default_name);
+  else gtk_widget_show(default_name);
+
   XMdcShowWidget(sfilew);
 
 }
@@ -428,24 +520,12 @@ void XMdcLutSelOpenCallbackOk(GtkWidget *w, GtkWidget *fs)
 
    fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
 
-   if (MdcLoadLUT(fname) == MDC_YES) {
-
-     sColormapSelection.CurMap = MDC_MAP_LOADED; 
-     MDC_COLOR_MAP = MDC_MAP_LOADED;
-
-     if (XMDC_FILE_OPEN == MDC_YES) XMdcApplyNewColorMap(MDC_COLOR_MAP);
-      
-   }else{
-
-     XMdcDisplayWarn("Could not read specified LUT file");
+   XMdcLoadLUT(fname);
  
-   }
-     
 }
 
 void XMdcLutSelOpen(GtkWidget *widget, gpointer data)
 {
-   char *XMEDCONLUT=NULL;
 
    if (olutw == NULL) {
 
@@ -472,8 +552,8 @@ void XMdcLutSelOpen(GtkWidget *widget, gpointer data)
 
    /* Lets set the filename, as if this were a save dialog, and we are giving
     *       a default filename */
-   if ((XMEDCONLUT = getenv("XMEDCONLUT")) != NULL) {
-     strncpy(xmdcstr,XMEDCONLUT,MDC_1KB_OFFSET);
+   if (XMEDCONLUT != NULL) {
+     strncpy(xmdcstr,XMEDCONLUT,MDC_1KB_OFFSET); xmdcstr[MDC_1KB_OFFSET]='\0';
 #ifdef MDC_DOS
      if (xmdcstr[strlen(xmdcstr)-1] != '\\' ) strcat(xmdcstr,"\\"); 
 #else
@@ -490,3 +570,122 @@ void XMdcLutSelOpen(GtkWidget *widget, gpointer data)
 
 }
 
+void XMdcRawPredefSelSaveCallbackOk(GtkWidget *widget, GtkWidget *fs)
+{
+  gchar *fname, *msg;
+
+  fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
+
+  if ((msg = MdcWritePredef(fname)) != NULL) {
+    XMdcDisplayWarn("%s",msg);
+  }else{
+    XMdcDisplayMesg("File successfully written");
+  }
+}
+
+void XMdcRawPredefSelSave(GtkWidget *widget, gpointer data)
+{
+  if (spredefw == NULL) {
+
+     /* Create a new file selection widget */
+     spredefw = gtk_file_selection_new ("Save Raw Predef File");
+
+     gtk_signal_connect(GTK_OBJECT(spredefw), "destroy",
+                        GTK_SIGNAL_FUNC(XMdcMedconQuit), NULL);
+
+     gtk_signal_connect(GTK_OBJECT(spredefw), "delete_event",
+                        GTK_SIGNAL_FUNC(XMdcHandlerToHide), NULL);
+
+     /* Connect the ok_button to save_ok_sel function */
+     gtk_signal_connect_object(
+        GTK_OBJECT(GTK_FILE_SELECTION (spredefw)->ok_button),
+        "clicked", GTK_SIGNAL_FUNC(gtk_widget_hide), GTK_OBJECT(spredefw));
+     gtk_signal_connect(
+        GTK_OBJECT(GTK_FILE_SELECTION (spredefw)->ok_button),
+        "clicked", GTK_SIGNAL_FUNC(XMdcRawPredefSelSaveCallbackOk),
+        GTK_OBJECT(spredefw));
+
+
+     /* Connect the cancel_button to hide the widget */
+     gtk_signal_connect_object(
+        GTK_OBJECT(GTK_FILE_SELECTION (spredefw)->cancel_button),
+        "clicked", GTK_SIGNAL_FUNC(gtk_widget_hide), GTK_OBJECT(spredefw));
+  }
+
+  if (XMEDCONRPI != NULL) {
+    strncpy(xmdcstr,XMEDCONRPI,MDC_1KB_OFFSET); xmdcstr[MDC_1KB_OFFSET]='\0';
+#ifdef MDC_DOS
+    if (xmdcstr[strlen(xmdcstr)-1] != '\\' ) strcat(xmdcstr,"\\");
+#else
+    if (xmdcstr[strlen(xmdcstr)-1] != '/' ) strcat(xmdcstr,"/");
+#endif
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(spredefw),xmdcstr);
+
+    gtk_file_selection_complete(GTK_FILE_SELECTION(spredefw),"predef.rpi");
+  }else{
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(spredefw),"predef.rpi");
+  }
+   
+  XMdcShowWidget(spredefw);
+
+}
+
+void XMdcRawPredefSelOpenCallbackOk(GtkWidget *w, GtkWidget *fs)
+{
+   char *fname, *msg;
+
+   fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
+
+   if (MdcCheckPredef(fname) == MDC_NO) {
+     XMdcDisplayWarn("Invalid raw predef input file"); return;
+   }
+
+   msg = MdcReadPredef(fname);
+   if (msg != NULL) XMdcDisplayWarn(msg);
+
+}
+
+void XMdcRawPredefSelOpen(GtkWidget *widget, gpointer data)
+{
+
+   if (opredefw == NULL) {
+
+     /* Create a new file selection widget */
+     opredefw = gtk_file_selection_new ("Load Raw Predef File");
+
+     gtk_signal_connect(GTK_OBJECT (opredefw), "destroy",
+                        GTK_SIGNAL_FUNC(XMdcMedconQuit), NULL);
+     gtk_signal_connect(GTK_OBJECT (opredefw), "delete_event",
+                       GTK_SIGNAL_FUNC(XMdcHandlerToHide), NULL);
+
+     /* Connect the ok_button to file_ok_sel function */
+     gtk_signal_connect_object(
+       GTK_OBJECT(GTK_FILE_SELECTION(opredefw)->ok_button),
+        "clicked", GTK_SIGNAL_FUNC(gtk_widget_hide), GTK_OBJECT(opredefw));
+     gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (opredefw)->ok_button),
+       "clicked", GTK_SIGNAL_FUNC(XMdcRawPredefSelOpenCallbackOk),opredefw);
+
+     /* Connect the cancel_button to hide the widget */
+     gtk_signal_connect_object(GTK_OBJECT
+       (GTK_FILE_SELECTION (opredefw)->cancel_button),
+        "clicked", GTK_SIGNAL_FUNC(gtk_widget_hide), GTK_OBJECT(opredefw));
+   }
+
+   if (XMEDCONRPI != NULL) {
+     strncpy(xmdcstr,XMEDCONRPI,MDC_1KB_OFFSET); xmdcstr[MDC_1KB_OFFSET]='\0';
+#ifdef MDC_DOS
+     if (xmdcstr[strlen(xmdcstr)-1] != '\\' ) strcat(xmdcstr,"\\");
+#else
+     if (xmdcstr[strlen(xmdcstr)-1] != '/' ) strcat(xmdcstr,"/");
+#endif
+     gtk_file_selection_set_filename(GTK_FILE_SELECTION(opredefw),xmdcstr);
+
+     gtk_file_selection_complete(GTK_FILE_SELECTION(opredefw),"*.rpi");
+   }else{
+     gtk_file_selection_set_filename(GTK_FILE_SELECTION(opredefw),"*.rpi");
+   }
+
+   XMdcShowWidget(opredefw);
+
+}
+
