@@ -8,11 +8,11 @@
  * project      : (X)MedCon by Erik Nolf                                   *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xfilesel.h,v 1.1.1.1 2000/10/28 16:51:46 enlf Exp $
+/* $Id: xfilesel.h,v 1.11 2004/04/06 11:32:02 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -34,19 +34,22 @@
 /****************************************************************************
                             F U N C T I O N S
 ****************************************************************************/
-
-void XMdcFileSelQuit(GtkWidget *widget, GtkWidget *fs);
-void XMdcFileSelCallbackOk(GtkWidget *w, guint data);
+void XMdcFileSelOpenCallbackOk(GtkWidget *w, GtkWidget *fs);
 void XMdcFileSelOpen(GtkWidget *widget, guint otype);
 void XMdcFileSelSaveCreateFormatMenu(GtkWidget *fs, guint format);
 void XMdcFileSelSaveCreateDefaultName(GtkWidget *fs);
-void XMdcFileSelSaveCallbackFormatMenu(GtkWidget *fs, Int8 *selected_format);
+void XMdcFileSelSaveCallbackFormatMenu(GtkWidget *fs, guint *selected_format);
+void XMdcFileSelSaveCallbackAlias(GtkObject *fs, char *filename);
 void XMdcFileSelSaveCallbackDefault(GtkObject *fs, char *filename);
 void XMdcFileSelSaveCallbackOk(GtkWidget *w, GtkWidget *fs);
 void XMdcFileSelSaveCallbackCancel(GtkWidget *w, GtkWidget *fs);
 void XMdcFileSelSave(GtkWidget *widget, guint format);
 void XMdcLutSelOpenCallbackOk(GtkWidget *w, GtkWidget *fs);
 void XMdcLutSelOpen(GtkWidget *w, gpointer data);
+void XMdcRawPredefSelSaveCallbackOk(GtkWidget *widget, GtkWidget *fs);
+void XMdcRawPredefSelSave(GtkWidget *widget, gpointer data);
+void XMdcRawPredefSelOpenCallbackOk(GtkWidget *w, GtkWidget *fs);
+void XMdcRawPredefSelOpen(GtkWidget *widget, gpointer data);
 
 #endif
 
