@@ -16,11 +16,11 @@
  *                XMdcImagesView()             - Show the images (viewer)  *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: ximages.c,v 1.1.1.1 2000/10/28 16:51:48 enlf Exp $
+/* $Id: ximages.c,v 1.17 2004/05/09 22:48:04 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -40,6 +40,8 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include <stdio.h>
+
 #include "xmedcon.h"
 
 /****************************************************************************
@@ -55,12 +57,12 @@
  * windows by clicking on places we wouldn't expect this to happen.
  *
  * So we 're blocking this button-press-event whenever we leave the 
- * corresponding drawing area (and ofcourse, allowing when entering)
+ * corresponding drawing area (and ofcourse, reallowing when entering)
  *
  * However, the gtk_signal_handler_block() function increments a number
  * instead of using a simple boolean. Whenever the "enter_notify_event"
  * is missed ... this number will never drop to zero again. And it does
- * happen when widgets come in front of our drawing are which are rebuild
+ * happen when widgets come in front of our drawing areas which are rebuild
  * because of a color correction or colormap change.
  *
  * Our SignalBlocker struct should prevent the gtk_signal_handler_block()
@@ -127,7 +129,8 @@ void XMdcImagesSetCursor(GtkWidget *widget, gpointer data)
 gboolean XMdcImagesCallbackExpose(GtkWidget *widget, GdkEventExpose *event, Uint32 *nr)
 {
   GdkGC *gc = widget->style->white_gc;
-  Uint32 i = (Uint32)(*nr), ex, ey, ew, eh, iw, ih, rw, rh;
+  Uint32 i = (Uint32)(*nr);
+  gint ex, ey, ew, eh, iw, ih, rw, rh;
 
   if ( my.im[i] == NULL ) return(TRUE);
 
@@ -155,8 +158,10 @@ gboolean XMdcImagesCallbackExpose(GtkWidget *widget, GdkEventExpose *event, Uint
     gdk_window_clear_area(widget->window, ex, ih, rw, rh);
   } 
 
-  if (sLabelSelection.CurState == MDC_YES) 
-    XMdcPrintImagesLabel(widget,i);
+  if (sLabelSelection.CurState == MDC_YES) {
+    XMdcPrintImageLabelIndex(widget,i);
+    XMdcPrintImageLabelTimes(widget,i);
+  }
 
   return(TRUE);
 }
@@ -187,14 +192,11 @@ void XMdcBuildCurrentImages(void)
 {
   Uint32 i, r, c, ri;
   Uint32 real_images;
+  float progress;
 
-  MdcDebugPrint("DEBUG: Build current images ...\n");
+  MdcDebugPrint("Building current images ...");
 
-  if (pages_to_frames == MDC_YES) {
-    real_images = my.fi->dim[3];
-  }else{
-    real_images = my.fi->number;
-  } 
+  real_images = XMdcPagesGetNrImages();
 
   /* get real_images_per_page and number of the beginning image */
   if (real_images < my.images_per_page) {
@@ -213,7 +215,7 @@ void XMdcBuildCurrentImages(void)
     }
   }
 
-  pvalue = 1./(float)(my.real_images_on_page + 1);
+  progress = 1./(float)(my.real_images_on_page + 1);
 
   /* build the images */
   i = my.startimage; ri=0;
@@ -224,7 +226,7 @@ void XMdcBuildCurrentImages(void)
      SignalBlocker *signal = (SignalBlocker *)&my.sblkr[ri];
 #endif
 
-     pvalue += 1./(float)(my.real_images_on_page + 1);
+     progress += 1./(float)(my.real_images_on_page + 1);
 
      if (ri == my.real_images_on_page) break; /* get OUT, got all images */
 
@@ -236,9 +238,9 @@ void XMdcBuildCurrentImages(void)
 
        my.image[ri] = gtk_drawing_area_new();
 
-       gtk_drawing_area_size(GTK_DRAWING_AREA(my.image[ri]),
-                             XMdcResize(my.fi->mwidth)  + XMDC_IMAGE_BORDER,
-                             XMdcResize(my.fi->mheight) + XMDC_IMAGE_BORDER);
+       gtk_drawing_area_size(GTK_DRAWING_AREA(my.image[ri])
+           ,(gint)(XMdcResize(my.fi->mwidth)  + XMDC_IMAGE_BORDER)
+           ,(gint)(XMdcResize(my.fi->mheight) + XMDC_IMAGE_BORDER));
 
        gtk_widget_set_events (my.image[ri], GDK_EXPOSURE_MASK
 		                          | GDK_BUTTON_PRESS_MASK
@@ -282,30 +284,38 @@ void XMdcBuildCurrentImages(void)
 
 
      if (GTK_WIDGET_VISIBLE(my.viewwindow)) {
-       /* update after each two rows */
-       if (((ri+1)%(my.images_horizontal<<1)) == 0) XMdcUpdateProgressBar(NULL);
+       if (my.images_horizontal < 10) {
+         /* update after each two rows */
+         if (((ri+1)%(my.images_horizontal*2)) == 0)
+           XMdcProgressBar(MDC_PROGRESS_SET,progress,NULL);
+       }else{
+         /* update after each five rows */
+         if (((ri+1)%(my.images_horizontal*5)) == 0)
+           XMdcProgressBar(MDC_PROGRESS_SET,progress,NULL);
+       }
      }else{
        /* update after each image */
-        XMdcUpdateProgressBar(NULL);
+        XMdcProgressBar(MDC_PROGRESS_SET,progress,NULL);
      }
   } 
 }
 
 void XMdcDisplayImages(void)
 {
-  XMdcBeginProgressBar("Preparing Viewer:");
+  XMdcProgressBar(MDC_PROGRESS_BEGIN,0.,"Preparing Viewer:");
   XMdcGetBoardDimensions();
   XMdcHandleBoardDimensions();
   XMdcBuildViewerWindow();
   XMdcBuildColorMap();
   XMdcBuildCurrentImages();
+  XMdcResizeNeeded();
   gtk_container_foreach(GTK_CONTAINER(my.imgstable),
                           (GtkCallback)XMdcImagesSetCursor,NULL);
-
 }
 
 void XMdcImagesView(GtkWidget *widget, gpointer data)
 {
+  if (XMdcNoFileOpened()) return;
   XMdcViewerShow();
 }
 
