@@ -8,19 +8,21 @@
  * project      : (X)MedCon by Erik Nolf                                   *
  *                                                                         *
  * Functions    : XMdcGetEcatLabelNumbers()     - Get ECAT label numbers   *
- *                XMdcGetImageLabel()           - Get image label          *
- *                XMdcPrintImagesLabel()        - Print the image labels   *
+ *                XMdcGetImageLabelIndex()      - Get image label index    *
+ *                XMdcGetImageLabelTimes()      - Get image label times    *
+ *                XMdcPrintImageLabelIndex()    - Print index labels       *
+ *                XMdcPrintImageLabelTimes()    - Print times labels       *
  *                XMdcLabelSelCallbackApply()   - Label Apply   callback   *
  *                XMdcUnsensitiveColNumFrames() - Set frames unusable      *
  *                XMdcSensitiveColNumFrames()   - Set frames   usable      *
  *                XMdcLabelSel()                - Label selection          *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xlabels.c,v 1.1.1.1 2000/10/28 16:51:51 enlf Exp $
+/* $Id: xlabels.c,v 1.16 2004/01/04 22:36:17 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -40,6 +42,18 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+#ifndef _WIN32
+#include <strings.h>
+#endif
+#endif
+
 #include "xmedcon.h"
 
 /****************************************************************************
@@ -83,82 +97,136 @@ void XMdcGetEcatLabelNumbers(Uint32 realnumber, Uint32 *plane, Uint32 *frame,
   *plane = p+1;                                           /* make 1-based */
 }
 
-char *XMdcGetImageLabel(Uint32 nr)
+char *XMdcGetImageLabelIndex(Uint32 nr)
 { 
    switch (sLabelSelection.CurStyle) {
-     case XMDC_LABEL_STYLE_ABS :   sprintf(labelname,"%u",my.realnumber[nr]+1);
+     case XMDC_LABEL_STYLE_ABS :   sprintf(labelindex,"%u",my.realnumber[nr]+1);
                                    break;
-     case XMDC_LABEL_STYLE_PAGE:   sprintf(labelname,"%u",nr+1);
+     case XMDC_LABEL_STYLE_PAGE:   sprintf(labelindex,"%u",nr+1);
                                    break;
      case XMDC_LABEL_STYLE_ECAT:
        {
           Uint32 p, f, g, b;
           XMdcGetEcatLabelNumbers(my.realnumber[nr],&p,&f,&g,&b);
-          sprintf(labelname,"%u.%u.%u.%u.%u",f,p,g,0,b); /* date = 0 default */
+          sprintf(labelindex,"%u.%u.%u.%u.%u",f,p,g,0,b); /* date = 0 default */
        }
        break;
    }
 
-   return(labelname);
+   return(labelindex);
 }
 
-void XMdcPrintImagesLabel(GtkWidget *widget, Uint32 nr)
+char *XMdcGetImageLabelTimes(Uint32 nr)
 {
-   char *label;
+  IMG_DATA *id;
+  Uint32 i, f;
+  float duration;
+  char s[10], d[10];
 
-  /* first, create a GC to draw on */
-   sLabelSelection.gc = gdk_gc_new(widget->window);
+  
+  if (my.fi->dynnr == 0) { labeltimes[0]='\0'; return(labeltimes); }
 
-   switch (sLabelSelection.CurColor) {
-     case XMDC_LABEL_RED   : sLabelSelection.color = &Red; break;
-     case XMDC_LABEL_GREEN : sLabelSelection.color = &Green; break;
-     case XMDC_LABEL_BLUE  : sLabelSelection.color = &Blue; break;
-     case XMDC_LABEL_YELLOW: sLabelSelection.color = &Yellow; break;
-   } 
+  i = my.realnumber[nr];
+  id = &my.fi->image[i];
 
-    
-  /* set the foreground to our color */
-   gdk_color_alloc(gtk_widget_get_colormap(widget), sLabelSelection.color);
-   gdk_gc_set_foreground(sLabelSelection.gc, sLabelSelection.color);
+  f = id->frame_number;
+  duration = MdcSingleImageDuration(my.fi,f-1);
+  strncpy(s,MdcGetStrHHMMSS(id->slice_start),10); s[9]='\0';
+  strncpy(d,MdcGetStrHHMMSS(duration),10);        s[9]='\0';
 
-   label=XMdcGetImageLabel(nr);
+  sprintf(labeltimes,"%u S=%s D=%s",f,s,d);
 
-   gdk_draw_string(widget->window, sfixed, sLabelSelection.gc, 2,
-                 XMdcResize(my.fi->mheight)-2, label);
+  return(labeltimes);
+}
 
-   gdk_gc_destroy(sLabelSelection.gc);
+
+
+void XMdcPrintImageLabelIndex(GtkWidget *widget, Uint32 nr)
+{
+	GdkColormap *map = gtk_widget_get_colormap(widget);
+	char *label;
+
+	/* first, create a GC to draw on */
+	sLabelSelection.gc = gdk_gc_new(widget->window);
+
+	switch (sLabelSelection.CurColor) {
+		case XMDC_LABEL_RED   : sLabelSelection.color = &Red; break;
+		case XMDC_LABEL_GREEN : sLabelSelection.color = &Green; break;
+		case XMDC_LABEL_BLUE  : sLabelSelection.color = &Blue; break;
+		case XMDC_LABEL_YELLOW: sLabelSelection.color = &Yellow; break;
+	} 
+
+
+	/* set the foreground to our color */
+	gdk_colormap_alloc_color(map,sLabelSelection.color, FALSE, TRUE);
+	gdk_gc_set_foreground(sLabelSelection.gc, sLabelSelection.color);
+
+	label=XMdcGetImageLabelIndex(nr);
+
+	gdk_draw_string(widget->window, sfixed, sLabelSelection.gc
+                        , 2, (gint)(XMdcResize(my.fi->mheight)-2), label);
+
+	gdk_colormap_free_colors(map,sLabelSelection.color,1);
+	gdk_gc_destroy(sLabelSelection.gc);
+}
+
+void XMdcPrintImageLabelTimes(GtkWidget *widget, Uint32 nr)
+{
+	GdkColormap *map = gtk_widget_get_colormap(widget);
+	char *label;
+
+	/* first, create a GC to draw on */
+	sLabelSelection.gc = gdk_gc_new(widget->window);
+
+	switch (sLabelSelection.CurColor) {
+		case XMDC_LABEL_RED   : sLabelSelection.color = &Red; break;
+		case XMDC_LABEL_GREEN : sLabelSelection.color = &Green; break;
+		case XMDC_LABEL_BLUE  : sLabelSelection.color = &Blue; break;
+		case XMDC_LABEL_YELLOW: sLabelSelection.color = &Yellow; break;
+	} 
+
+
+	/* set the foreground to our color */
+	gdk_colormap_alloc_color(map,sLabelSelection.color, FALSE, TRUE);
+	gdk_gc_set_foreground(sLabelSelection.gc, sLabelSelection.color);
+
+	label=XMdcGetImageLabelTimes(nr);
+
+	gdk_draw_string(widget->window, sfixed, sLabelSelection.gc, 2, 10, label);
+
+	gdk_colormap_free_colors(map,sLabelSelection.color,1);
+	gdk_gc_destroy(sLabelSelection.gc);
 }
 
 void XMdcLabelSelCallbackApply(GtkWidget *widget, gpointer data)
 {
-   gint color=XMDC_LABEL_YELLOW, state=MDC_YES, style=XMDC_LABEL_STYLE_PAGE;
-   MdcDebugPrint("DEBUG: label = ");
-   if (GTK_TOGGLE_BUTTON(sLabelSelection.On)->active) {
-     MdcDebugPrint("ON "); state = MDC_YES;
-   }else if (GTK_TOGGLE_BUTTON(sLabelSelection.Off)->active) {
-     MdcDebugPrint("OFF "); state = MDC_NO;
-   }
+	gint color=XMDC_LABEL_YELLOW, state=MDC_YES, style=XMDC_LABEL_STYLE_PAGE;
+	if (GTK_TOGGLE_BUTTON(sLabelSelection.On)->active) {
+		MdcDebugPrint("labels: ON "); state = MDC_YES;
+	}else if (GTK_TOGGLE_BUTTON(sLabelSelection.Off)->active) {
+		MdcDebugPrint("labels: OFF "); state = MDC_NO;
+	}
 
-   if (GTK_TOGGLE_BUTTON(sLabelSelection.Red)->active) {
-     MdcDebugPrint("Red"); color = XMDC_LABEL_RED;
-   }else if (GTK_TOGGLE_BUTTON(sLabelSelection.Green)->active) {
-     MdcDebugPrint("Green"); color = XMDC_LABEL_GREEN;
-   }else if (GTK_TOGGLE_BUTTON(sLabelSelection.Blue)->active) {
-     MdcDebugPrint("Blue"); color = XMDC_LABEL_BLUE;
-   }else if (GTK_TOGGLE_BUTTON(sLabelSelection.Yellow)->active) {
-     MdcDebugPrint("Yellow"); color = XMDC_LABEL_YELLOW;
-   }
+	MdcDebugPrint("label color: ");
+	if (GTK_TOGGLE_BUTTON(sLabelSelection.Red)->active) {
+		MdcDebugPrint("\tred"); color = XMDC_LABEL_RED;
+	}else if (GTK_TOGGLE_BUTTON(sLabelSelection.Green)->active) {
+		MdcDebugPrint("\tgreen"); color = XMDC_LABEL_GREEN;
+	}else if (GTK_TOGGLE_BUTTON(sLabelSelection.Blue)->active) {
+		MdcDebugPrint("\tblue"); color = XMDC_LABEL_BLUE;
+	}else if (GTK_TOGGLE_BUTTON(sLabelSelection.Yellow)->active) {
+		MdcDebugPrint("\tyellow"); color = XMDC_LABEL_YELLOW;
+	}
 
-   if (GTK_TOGGLE_BUTTON(sLabelSelection.NrAbsolute)->active) {
-     MdcDebugPrint(" Absolute"); style = XMDC_LABEL_STYLE_ABS;
+	MdcDebugPrint("label number: ");
+	if (GTK_TOGGLE_BUTTON(sLabelSelection.NrAbsolute)->active) {
+		MdcDebugPrint("\tabsolute"); style = XMDC_LABEL_STYLE_ABS;
    }else if (GTK_TOGGLE_BUTTON(sLabelSelection.NrInPage)->active) {
-     MdcDebugPrint(" In Page");  style = XMDC_LABEL_STYLE_PAGE;
+     MdcDebugPrint("\tin page");  style = XMDC_LABEL_STYLE_PAGE;
    }else if (GTK_TOGGLE_BUTTON(sLabelSelection.NrEcat)->active) {
-     MdcDebugPrint(" Ecat");     style = XMDC_LABEL_STYLE_ECAT;
+     MdcDebugPrint("\tecat");     style = XMDC_LABEL_STYLE_ECAT;
    }
 
-   MdcDebugPrint("\n"); 
-
    if (state!=sLabelSelection.CurState || color!=sLabelSelection.CurColor ||
        style!=sLabelSelection.CurStyle) {
      sLabelSelection.CurState = state;
