@@ -16,11 +16,11 @@
  *                XMdcViewerDisableAutoShrink()- Disable auto shrinking    *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xviewer.c,v 1.1.1.1 2000/10/28 16:52:02 enlf Exp $
+/* $Id: xviewer.c,v 1.21 2004/01/04 22:36:17 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -40,6 +40,14 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#include <math.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
 #include "xmedcon.h"
 
 /****************************************************************************
@@ -54,11 +62,7 @@ int XMdcGetBoardDimensions(void)
    Uint32 trial_images_vertical = 0, trial_images_horizontal = 0;
    int    FOUND = MDC_NO;
 
-   if (pages_to_frames == MDC_YES) {
-     real_images = my.fi->dim[3];
-   }else{
-     real_images = my.fi->number;
-   }
+   real_images = XMdcPagesGetNrImages();
  
    if (real_images == 0)
      XMdcDisplayFatalErr(MDC_BAD_CODE,"Number of images is zero !?");
@@ -174,16 +178,14 @@ int XMdcGetBoardDimensions(void)
    my.images_per_page   = arbitrary_images_per_page;
    my.number_of_pages   = arbitrary_pages;
 
-   /* we still have to take care of pages_to_frames */
-   if (pages_to_frames == MDC_YES)
-     if (my.fi->dim[3] < my.images_per_page ) {
-       my.images_per_page = my.fi->dim[3];
-       my.number_of_pages  = (my.fi->number+(my.images_per_page - 1))
-                             / my.images_per_page;
-
-       my.images_vertical  = (my.images_per_page + (my.images_horizontal - 1))
-                             / my.images_horizontal; 
+   /* we still have to take care of less images per page */
+   if (XMdcPagesGetNrImages() < my.images_per_page ) {
+     my.images_per_page = XMdcPagesGetNrImages();
+     my.number_of_pages  = (my.fi->number+(my.images_per_page - 1))
+                           / my.images_per_page;
 
+     my.images_vertical  = (my.images_per_page + (my.images_horizontal - 1))
+                           / my.images_horizontal; 
    }
 
    if ((my.fi->number < my.images_per_page) && (my.images_vertical == 1)) {
@@ -218,7 +220,7 @@ void XMdcHandleBoardDimensions(void)
   if (vertical*(XMdcResize(h)+(XMDC_IMAGE_BORDER<<1)) < XMDC_COLORMAP_HEIGHT) {
     my.cmap_h = XMDC_COLORMAP_HEIGHT;
   }else{
-    my.cmap_h = vertical * (XMdcResize(h)+XMDC_IMAGE_BORDER);
+    my.cmap_h = vertical * (XMdcResize(h) + XMDC_IMAGE_BORDER);
   }
   my.cmap_w = XMDC_COLORMAP_WIDTH;
 
@@ -286,20 +288,20 @@ void XMdcHandleBoardDimensions(void)
 
   for (i = 0; i<my.number_of_pages; i++) my.pagenumber[i] = i;
 
-  MdcDebugPrint("DEBUG: screen_width      = %d\n",gdk_screen_width());
-  MdcDebugPrint("DEBUG: screen_height     = %d\n",gdk_screen_height());
-  MdcDebugPrint("DEBUG: mwidth            = %u\n",my.fi->mwidth);
-  MdcDebugPrint("DEBUG: mheight           = %u\n",my.fi->mheight);
-  MdcDebugPrint("DEBUG: resize            = %d\n",sResizeSelection.CurType);
-  MdcDebugPrint("DEBUG: fi.number         = %u\n",my.fi->number);
-  MdcDebugPrint("DEBUG: fi.dim[3]         = %u\n",my.fi->dim[3]); 
-  MdcDebugPrint("DEBUG: curpage           = %u\n",my.curpage);
-  MdcDebugPrint("DEBUG: images_horizontal = %u\n",my.images_horizontal);
-  MdcDebugPrint("DEBUG: images_vertical   = %u\n",my.images_vertical);
-  MdcDebugPrint("DEBUG: images_per_page   = %u\n",my.images_per_page);
-  MdcDebugPrint("DEBUG: number_of_pages   = %u\n",my.number_of_pages);
-  MdcDebugPrint("DEBUG: cmap_w            = %u\n",my.cmap_w);
-  MdcDebugPrint("DEBUG: cmap_h            = %u\n",my.cmap_h); 
+  MdcDebugPrint("screen_width      = %d",gdk_screen_width());
+  MdcDebugPrint("screen_height     = %d",gdk_screen_height());
+  MdcDebugPrint("mwidth            = %u",my.fi->mwidth);
+  MdcDebugPrint("mheight           = %u",my.fi->mheight);
+  MdcDebugPrint("resize            = %d",sResizeSelection.CurType);
+  MdcDebugPrint("fi.number         = %u",my.fi->number);
+  MdcDebugPrint("fi.dim[3]         = %u",my.fi->dim[3]); 
+  MdcDebugPrint("curpage           = %u",my.curpage);
+  MdcDebugPrint("images_horizontal = %u",my.images_horizontal);
+  MdcDebugPrint("images_vertical   = %u",my.images_vertical);
+  MdcDebugPrint("images_per_page   = %u",my.images_per_page);
+  MdcDebugPrint("number_of_pages   = %u",my.number_of_pages);
+  MdcDebugPrint("cmap_w            = %u",my.cmap_w);
+  MdcDebugPrint("cmap_h            = %u",my.cmap_h); 
 
 }
 
@@ -307,29 +309,33 @@ void XMdcBuildViewerWindow(void)
 {
   GtkWidget *box1;
   GtkWidget *box2;
+  GtkWidget *vbox;
+  GtkWidget *label;
   GtkWidget *frame;
   GtkWidget *winbox;
-  GtkWidget *imgsbox;
   GtkWidget *button;
+  GtkWidget *spinner;
+  GtkWidget *entrybox;
   GtkWidget *separator;
+  GtkAdjustment *adj;
 
   if (my.viewwindow == NULL) {
-    my.viewwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    my.viewwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_policy(GTK_WINDOW(my.viewwindow),FALSE,FALSE,FALSE);
 
-    gtk_signal_connect (GTK_OBJECT (my.viewwindow), "delete-event",
-                        GTK_SIGNAL_FUNC(XMdcViewerHide), NULL);
-    gtk_signal_connect (GTK_OBJECT (my.viewwindow), "destroy",
-                        GTK_SIGNAL_FUNC(XMdcMedconQuit), NULL);
-    gtk_container_set_border_width (GTK_CONTAINER (my.viewwindow), 0);
+    gtk_signal_connect(GTK_OBJECT(my.viewwindow), "delete-event",
+                       GTK_SIGNAL_FUNC(XMdcViewerHide), NULL);
+    gtk_signal_connect(GTK_OBJECT(my.viewwindow), "destroy",
+                       GTK_SIGNAL_FUNC(XMdcMedconQuit), NULL);
+    gtk_container_set_border_width(GTK_CONTAINER (my.viewwindow), 0);
   }
 
-  gtk_window_set_title (GTK_WINDOW (my.viewwindow), my.fi->ifname);
+  gtk_window_set_title(GTK_WINDOW(my.viewwindow), my.fi->ifname);
 
   gtk_widget_realize(my.viewwindow);
 
-  box1 = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (my.viewwindow), box1);
+  box1 = gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER (my.viewwindow), box1);
   gtk_widget_show(box1);
 
   my.viewbox = box1;
@@ -343,73 +349,147 @@ void XMdcBuildViewerWindow(void)
   gtk_box_pack_start(GTK_BOX(box2),my.pagemenu,TRUE,TRUE,0); 
   gtk_widget_show(my.pagemenu);
 
-  if (my.number_of_pages > 1) {
-    button = gtk_button_new_with_label("Next");
-    gtk_signal_connect(GTK_OBJECT(button),"clicked",
-                       GTK_SIGNAL_FUNC(XMdcPagesNext),NULL);
-    gtk_box_pack_start(GTK_BOX(box2),button,TRUE,TRUE,0);
-    gtk_widget_show(button);
-
-    button = gtk_button_new_with_label("Prev");
-    gtk_signal_connect(GTK_OBJECT(button),"clicked",
-                       GTK_SIGNAL_FUNC(XMdcPagesPrev),NULL);
-    gtk_box_pack_start(GTK_BOX(box2),button,TRUE,TRUE,0);
-    gtk_widget_show(button);
-  }
-  /* layout for images and colormap */
-   frame = gtk_frame_new(NULL);
-   gtk_container_set_border_width(GTK_CONTAINER(frame),4);
-   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
-   gtk_box_pack_start(GTK_BOX(box1),frame,TRUE,TRUE,0);
-   gtk_widget_show(frame);
-
-   winbox = gtk_hbox_new(FALSE, 0);
-   gtk_container_add(GTK_CONTAINER(frame),winbox);
-   gtk_widget_show(winbox);
-
-   imgsbox = gtk_hbox_new(TRUE,0);
-   gtk_container_set_border_width(GTK_CONTAINER(imgsbox),0);
-   gtk_box_pack_start(GTK_BOX(winbox),imgsbox,TRUE,TRUE,0);
-   gtk_widget_show(imgsbox);
-
-   my.imgstable=gtk_table_new(my.images_vertical,my.images_horizontal,TRUE);
-   gtk_box_pack_start(GTK_BOX(imgsbox), my.imgstable, TRUE, TRUE, 0);
-   gtk_widget_show(my.imgstable);
-
-   separator = gtk_vseparator_new ();
-   gtk_box_pack_start (GTK_BOX (winbox), separator, FALSE, FALSE, 0);
-   gtk_widget_show (separator);
-
-   my.cmapbox = gtk_event_box_new();
-   gtk_box_pack_start(GTK_BOX(winbox),my.cmapbox,FALSE,FALSE,0);
-   gtk_widget_show(my.cmapbox);
-
-  /* seperator */
-  separator = gtk_hseparator_new ();
-  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
-
-  /* layout for buttons */      
-  box2 = gtk_hbox_new (FALSE, 2);
-  gtk_container_set_border_width (GTK_CONTAINER (box2), 0);
-  gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
-  gtk_widget_show(box2);
- 
-  button = gtk_button_new_with_label ("Hide");
-  gtk_signal_connect(GTK_OBJECT (button), "clicked",
-                     GTK_SIGNAL_FUNC (XMdcViewerHide), NULL);
-  gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
+  button = gtk_button_new_with_label("Next");
+  gtk_signal_connect(GTK_OBJECT(button),"clicked",
+                     GTK_SIGNAL_FUNC(XMdcPagesNext),NULL);
+  gtk_box_pack_start(GTK_BOX(box2),button,TRUE,TRUE,0);
   gtk_widget_show(button);
 
-  button = gtk_button_new_with_label ("Labels");
-  gtk_signal_connect(GTK_OBJECT (button), "clicked",
-                     GTK_SIGNAL_FUNC (XMdcLabelSel), NULL);
-  gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
+  button = gtk_button_new_with_label("Prev");
+  gtk_signal_connect(GTK_OBJECT(button),"clicked",
+                     GTK_SIGNAL_FUNC(XMdcPagesPrev),NULL);
+  gtk_box_pack_start(GTK_BOX(box2),button,TRUE,TRUE,0);
+  gtk_widget_show(button);
+
+  /* layout for images and colormap */
+  frame = gtk_frame_new(NULL);
+  gtk_container_set_border_width(GTK_CONTAINER(frame),4);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
+  gtk_box_pack_start(GTK_BOX(box1),frame,TRUE,TRUE,0);
+  gtk_widget_show(frame);
+
+  winbox = gtk_hbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(frame),winbox);
+  gtk_widget_show(winbox);
+
+  my.imgsbox = gtk_hbox_new(TRUE,0);
+  gtk_container_set_border_width(GTK_CONTAINER(my.imgsbox),0);
+  gtk_box_pack_start(GTK_BOX(winbox),my.imgsbox,TRUE,TRUE,0);
+  gtk_widget_show(my.imgsbox);
+
+  my.imgstable=gtk_table_new(my.images_vertical,my.images_horizontal,TRUE);
+  gtk_box_pack_start(GTK_BOX(my.imgsbox), my.imgstable, TRUE, TRUE, 0);
+  gtk_widget_show(my.imgstable);
+
+  separator = gtk_vseparator_new();
+  gtk_box_pack_start(GTK_BOX(winbox), separator, FALSE, FALSE, 0);
+  gtk_widget_show(separator);
+
+  my.cmapbox = gtk_event_box_new();
+  gtk_box_pack_start(GTK_BOX(winbox),my.cmapbox,FALSE,FALSE,0);
+  gtk_widget_show(my.cmapbox);
+
+  XMdcApplyMapPlace(XMDC_CMAP_PLACE);
+
+  /* seperator */
+  separator = gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(box1), separator, FALSE, TRUE, 0);
+
+  /* layout for buttons */  
+
+  vbox = gtk_vbox_new(FALSE, 2);
+  gtk_container_set_border_width(GTK_CONTAINER(box2), 0);
+  gtk_box_pack_start(GTK_BOX(box1), vbox, FALSE, TRUE, 0);
+  gtk_widget_show(vbox);
+
+  box2 = gtk_hbox_new(FALSE, 2);
+  gtk_container_set_border_width(GTK_CONTAINER(box2), 0);
+  gtk_box_pack_start(GTK_BOX(vbox), box2, FALSE, FALSE, 0);
+  gtk_widget_show(box2);
+
+
+  button = gtk_button_new_with_label("Toggle Entries");
+  gtk_box_pack_start(GTK_BOX(box2),button,TRUE,TRUE,0);
+  gtk_widget_show(button);
+
+  entrybox = gtk_hbox_new(FALSE, 2);
+  gtk_container_set_border_width(GTK_CONTAINER(box2), 0);
+  gtk_box_pack_start(GTK_BOX(box2), entrybox, FALSE, FALSE, 0);
+  gtk_widget_show(entrybox);
+
+  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+                            GTK_SIGNAL_FUNC(XMdcToggleVisibility),
+                            GTK_OBJECT(entrybox));
+
+  label = gtk_label_new("Page: ");
+  gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_RIGHT);
+  gtk_widget_set_name(label,"FixedLabel");
+  gtk_box_pack_start(GTK_BOX(entrybox),label,FALSE,TRUE,5);
+  gtk_widget_show(label);
+
+  adj=(GtkAdjustment *)gtk_adjustment_new(1., 1., (float)my.number_of_pages,
+                                          1., 5., 0.);
+
+  spinner = gtk_spin_button_new(adj, 0.0, 0);
+  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinner), TRUE);
+  gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spinner),
+                                  GTK_SHADOW_ETCHED_IN);
+  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinner), TRUE);
+  gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(spinner), TRUE);
+
+  gtk_box_pack_start(GTK_BOX(entrybox),spinner,FALSE,FALSE,5);
+
+  gtk_signal_connect_after(GTK_OBJECT(spinner),"key_press_event",
+                     GTK_SIGNAL_FUNC(XMdcPagesGoTo),NULL);
+  gtk_signal_connect_after(GTK_OBJECT(spinner), "button_release_event",
+                           GTK_SIGNAL_FUNC(XMdcPagesGoTo),NULL);
+  gtk_widget_show(spinner);
+
+  label = gtk_label_new("Table: ");
+  gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_RIGHT);
+  gtk_widget_set_name(label,"FixedLabel");
+  gtk_box_pack_start(GTK_BOX(entrybox),label,FALSE,TRUE,5);
+  gtk_widget_show(label);
+
+  adj=(GtkAdjustment *)gtk_adjustment_new(1., 1., (float)XMDC_MAX_LOADABLE_LUTS,
+                                          1., 5., 0.);
+
+  spinner = gtk_spin_button_new(adj, 0.0, 0);
+  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinner), TRUE);
+  gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spinner), 
+                                  GTK_SHADOW_ETCHED_IN);
+  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinner), TRUE);
+  gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(spinner), TRUE);
+
+  gtk_box_pack_start(GTK_BOX(entrybox),spinner,FALSE,FALSE,5);
+
+  gtk_signal_connect_after(GTK_OBJECT(spinner),"key_press_event",
+                     GTK_SIGNAL_FUNC(XMdcChangeLUT),NULL);
+  gtk_signal_connect_after(GTK_OBJECT(spinner), "button_release_event",
+                           GTK_SIGNAL_FUNC(XMdcChangeLUT),NULL);
+  gtk_widget_show(spinner);
+
+  box2 = gtk_hbox_new(FALSE, 2);
+  gtk_container_set_border_width(GTK_CONTAINER(box2), 0);
+  gtk_box_pack_start(GTK_BOX(vbox), box2, FALSE, TRUE, 0);
+  gtk_widget_show(box2);
+ 
+  button = gtk_button_new_with_label(" Hide ");
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(XMdcViewerHide), NULL);
+  gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+
+  button = gtk_button_new_with_label("Labels");
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(XMdcLabelSel), NULL);
+  gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
   gtk_widget_show(button);
 
   button = gtk_button_new_with_label("Render");
-  gtk_signal_connect(GTK_OBJECT (button), "clicked",
-                     GTK_SIGNAL_FUNC (XMdcRenderingSel), NULL);
-  gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(XMdcRenderingSel), NULL);
+  gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
   gtk_widget_show(button);
 
 }
@@ -423,17 +503,18 @@ void XMdcViewerHide(void)
 
 void XMdcViewerShow(void)
 {
-  if (XMdcNoFileOpened()) return;
+  if (XMDC_FILE_OPEN == MDC_NO) return;
   if (my.viewwindow != NULL) {
-    MdcDebugPrint("DEBUG: Show viewwindow\n");
+    MdcDebugPrint("Show viewer window ...");
     gtk_widget_show(my.viewwindow);
   }
+  XMdcViewerDisableAutoShrink();
 }
 
 void XMdcViewerEnableAutoShrink(void)
 {
   if (my.viewwindow != NULL) {
-    MdcDebugPrint("DEBUG: Enable AutoShrink\n");
+    MdcDebugPrint("enable auto shrink");
     gtk_window_set_policy(GTK_WINDOW(my.viewwindow),TRUE,TRUE,TRUE);
   }
 }
@@ -441,7 +522,7 @@ void XMdcViewerEnableAutoShrink(void)
 void XMdcViewerDisableAutoShrink(void)
 {
   if (my.viewwindow != NULL) {
-    MdcDebugPrint("DEBUG: Disable AutoShrink\n");
+    MdcDebugPrint("disable auto shrink");
     gtk_window_set_policy(GTK_WINDOW(my.viewwindow),FALSE,FALSE,FALSE);
   }
 }
