@@ -13,11 +13,11 @@
  *                XMdcImagesZoom()                - Display zoomed image   *
  *                                                                         *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* $Id: xzoom.c,v 1.1.1.1 2000/10/28 16:52:04 enlf Exp $
+/* $Id: xzoom.c,v 1.10 2004/01/04 22:36:17 enlf Exp $
  */
 
 /*
-   Copyright (C) 1997-2000 by Erik Nolf
+   Copyright (C) 1997-2004 by Erik Nolf
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -37,6 +37,13 @@
                               H E A D E R S
 ****************************************************************************/
 
+#include "m-depend.h"
+
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
 #include "xmedcon.h"
 
 /****************************************************************************
@@ -58,15 +65,15 @@ typedef struct ZoomStruct_t {
 ****************************************************************************/
 
 #ifdef _WIN32
-static void Win32MinDim128(ZoomStruct *zoom, Uint32 *arg_w, Uint32 *arg_h)
+static void Win32MinDim128(ZoomStruct *zoom, int *arg_w, int *arg_h)
 {
-  Uint32 w=*arg_w, h=*arg_h;
+  int w=*arg_w, h=*arg_h;
   float factor = zoom->factor;
 
   while (w < 128 || h < 128) {
     factor *= (float)XMDC_ZOOM_FACTOR;
-    w = (Uint32)((float)w * (float)XMDC_ZOOM_FACTOR);
-    h = (Uint32)((float)h * (float)XMDC_ZOOM_FACTOR);
+    w = (int)((float)w * (float)XMDC_ZOOM_FACTOR);
+    h = (int)((float)h * (float)XMDC_ZOOM_FACTOR);
   }
   zoom->factor = factor;
   *arg_w = w;
@@ -152,14 +159,14 @@ void XMdcImagesZoomIn(GtkWidget *window)
 {
   GdkPixbuf *new;
   ZoomStruct *zoom;
-  Uint32 w, h;
+  int w, h;
 
   zoom = (ZoomStruct *)gtk_object_get_data(GTK_OBJECT(window),"zoomstruct");
 
   zoom->factor *= (float) XMDC_ZOOM_FACTOR;
 
-  w = (Uint32)((float)gdk_pixbuf_get_width(zoom->im)  * zoom->factor); 
-  h = (Uint32)((float)gdk_pixbuf_get_height(zoom->im) * zoom->factor);
+  w = (int)((float)gdk_pixbuf_get_width(zoom->im)  * zoom->factor); 
+  h = (int)((float)gdk_pixbuf_get_height(zoom->im) * zoom->factor);
 
 #ifdef _WIN32
   Win32MinDim128(zoom, &w, &h);
@@ -194,7 +201,7 @@ void XMdcImagesZoomOut(GtkWidget *window)
 {
   GdkPixbuf *new;
   ZoomStruct *zoom;
-  Uint32 w, h;
+  int w, h;
 
   zoom = (ZoomStruct *)gtk_object_get_data(GTK_OBJECT(window),"zoomstruct");
 
@@ -202,8 +209,8 @@ void XMdcImagesZoomOut(GtkWidget *window)
 
     zoom->factor /= (float)XMDC_ZOOM_FACTOR;
 
-    w = (Uint32)((float)gdk_pixbuf_get_width(zoom->im)  * zoom->factor);
-    h = (Uint32)((float)gdk_pixbuf_get_height(zoom->im) * zoom->factor);
+    w = (int)((float)gdk_pixbuf_get_width(zoom->im)  * zoom->factor);
+    h = (int)((float)gdk_pixbuf_get_height(zoom->im) * zoom->factor);
 
 #ifdef _WIN32
     Win32MinDim128(zoom, &w, &h);
@@ -246,7 +253,7 @@ void XMdcImagesZoom(GtkWidget *widget, Uint32 nr)
   GtkWidget *tblbox;
   GtkWidget *darea;
   ZoomStruct *zoom=NULL;
-  Uint32 w, h, fctr;
+  int w, h, fctr;
 
 
   /* allocate structure */
@@ -262,13 +269,13 @@ void XMdcImagesZoom(GtkWidget *widget, Uint32 nr)
   }
 
   if (zoom->factor < 1.) {
-    fctr = (Uint32)(1./zoom->factor);
-    w = (my.fi->image[zoom->nr].width  + (fctr-1)) / fctr;
-    h = (my.fi->image[zoom->nr].height + (fctr-1)) / fctr;
+    fctr = (int)1./zoom->factor;
+    w = ((int)my.fi->image[zoom->nr].width  + (fctr-1)) / fctr;
+    h = ((int)my.fi->image[zoom->nr].height + (fctr-1)) / fctr;
   }else{
-    fctr = (Uint32)zoom->factor;
-    w = my.fi->image[zoom->nr].width  * fctr;
-    h = my.fi->image[zoom->nr].height * fctr;
+    fctr = (int)zoom->factor;
+    w = (int)my.fi->image[zoom->nr].width  * fctr;
+    h = (int)my.fi->image[zoom->nr].height * fctr;
   }
 
 #ifdef _WIN32
